import React from "react"
import cx from "classnames"
import Im, { List, Seq, Iterable } from "immutable"
import XML from "xml"
import memoizee from "memoizee"

function isObject(obj) {
    return !!obj && typeof obj === "object"
}

function objectify (thing) {
    if(!isObject(thing))
        return {}
    if(isImmutable(thing))
        return thing.toObject()
    return thing
}

function isFunc(thing) {
    return typeof(thing) === "function"
}

function normalizeArray(arr) {
    if(Array.isArray(arr))
        return arr
    return [arr]
}

const primitives = {
    "string": () => "string",
    "string_email": () => "user@example.com",
    "string_date-time": () => new Date().toISOString(),
    "number": () => 0,
    "number_float": () => 0.0,
    "integer": () => 0,
    "boolean": (schema) => typeof schema.default === "boolean" ? schema.default : true
}

const primitive = (schema) => {
    schema = objectify(schema)
    let { type, format } = schema

    let fn = primitives[`${type}_${format}`] || primitives[type]

    if(isFunc(fn))
        return fn(schema)

    return "Unknown Type: " + schema.type
}


export const sampleFromSchema = (schema, config={}) => {
    let { type, example, properties, additionalProperties, items } = objectify(schema)
    let { includeReadOnly, includeWriteOnly } = config

    if(example !== undefined)
        return example

    if(!type) {
        if(properties) {
            type = "object"
        } else if(items) {
            type = "array"
        } else {
            return
        }
    }

    if(type === "object") {
        let props = objectify(properties)
        let obj = {}
        for (var name in props) {
            if ( props[name].readOnly && !includeReadOnly ) {
                continue
            }
            if ( props[name].writeOnly && !includeWriteOnly ) {
                continue
            }
            obj[name] = sampleFromSchema(props[name], config)
        }

        if ( additionalProperties === true ) {
            obj.additionalProp1 = {}
        } else if ( additionalProperties ) {
            let additionalProps = objectify(additionalProperties)
            let additionalPropVal = sampleFromSchema(additionalProps, config)

            for (let i = 1; i < 4; i++) {
                obj["additionalProp" + i] = additionalPropVal
            }
        }
        return obj
    }

    if(type === "array") {
        return [ sampleFromSchema(items, config) ]
    }

    if(schema["enum"]) {
        if(schema["default"])
            return schema["default"]
        return normalizeArray(schema["enum"])[0]
    }

    if (type === "file") {
        return
    }

    return primitive(schema)
}

export const inferSchema = (thing) => {
    if(thing.schema)
        thing = thing.schema

    if(thing.properties) {
        thing.type = "object"
    }

    return thing // Hopefully this will have something schema like in it... `type` for example
}


export const sampleXmlFromSchema = (schema, config={}) => {
    let objectifySchema = objectify(schema)
    let { type, properties, additionalProperties, items, example } = objectifySchema
    let { includeReadOnly, includeWriteOnly } = config
    let defaultValue = objectifySchema.default
    let res = {}
    let _attr = {}
    let { xml } = schema
    let { name, prefix, namespace } = xml
    let enumValue = objectifySchema.enum
    let displayName, value

    if(!type) {
        if(properties || additionalProperties) {
            type = "object"
        } else if(items) {
            type = "array"
        } else {
            return
        }
    }

    name = name || "notagname"
    // add prefix to name if exists
    displayName = (prefix ? prefix + ":" : "") + name
    if ( namespace ) {
        //add prefix to namespace if exists
        let namespacePrefix = prefix ? ( "xmlns:" + prefix ) : "xmlns"
        _attr[namespacePrefix] = namespace
    }

    if (type === "array") {
        if (items) {
            items.xml = items.xml || xml || {}
            items.xml.name = items.xml.name || xml.name

            if (xml.wrapped) {
                res[displayName] = []
                if (Array.isArray(example)) {
                    example.forEach((v)=>{
                        items.example = v
                        res[displayName].push(sampleXmlFromSchema(items, config))
                    })
                } else if (Array.isArray(defaultValue)) {
                    defaultValue.forEach((v)=>{
                        items.default = v
                        res[displayName].push(sampleXmlFromSchema(items, config))
                    })
                } else {
                    res[displayName] = [sampleXmlFromSchema(items, config)]
                }

                if (_attr) {
                    res[displayName].push({_attr: _attr})
                }
                return res
            }

            let _res = []

            if (Array.isArray(example)) {
                example.forEach((v)=>{
                    items.example = v
                    _res.push(sampleXmlFromSchema(items, config))
                })
                return _res
            } else if (Array.isArray(defaultValue)) {
                defaultValue.forEach((v)=>{
                    items.default = v
                    _res.push(sampleXmlFromSchema(items, config))
                })
                return _res
            }

            return sampleXmlFromSchema(items, config)
        }
    }

    if (type === "object") {
        let props = objectify(properties)
        res[displayName] = []
        example = example || {}

        for (let propName in props) {
            if ( props[propName].readOnly && !includeReadOnly ) {
                continue
            }
            if ( props[propName].writeOnly && !includeWriteOnly ) {
                continue
            }

            props[propName].xml = props[propName].xml || {}

            if (props[propName].xml.attribute) {
                let enumAttrVal = Array.isArray(props[propName].enum) && props[propName].enum[0]
                let attrExample = props[propName].example
                let attrDefault = props[propName].default
                _attr[props[propName].xml.name || propName] = attrExample!== undefined && attrExample
                    || example[propName] !== undefined && example[propName] || attrDefault !== undefined && attrDefault
                    || enumAttrVal || primitive(props[propName])
            } else {
                props[propName].xml.name = props[propName].xml.name || propName
                props[propName].example = props[propName].example !== undefined ? props[propName].example : example[propName]
                let t = sampleXmlFromSchema(props[propName])
                if (Array.isArray(t)) {
                    res[displayName] = res[displayName].concat(t)
                } else {
                    res[displayName].push(t)
                }

            }
        }

        if (additionalProperties === true) {
            res[displayName].push({additionalProp: "Anything can be here"})
        } else if (additionalProperties) {
            res[displayName].push({additionalProp: primitive(additionalProperties)})
        }

        if (_attr) {
            res[displayName].push({_attr: _attr})
        }
        return res
    }

    if (example !== undefined) {
        value = example
    } else if (defaultValue !== undefined) {
        //display example if exists
        value = defaultValue
    } else if (Array.isArray(enumValue)) {
        //display enum first value
        value = enumValue[0]
    } else {
        //set default value
        value = primitive(schema)
    }

    res[displayName] = _attr ? [{_attr: _attr}, value] : value

    return res
}

export function createXMLExample(schema, config) {
    let json = sampleXmlFromSchema(schema, config)
    if (!json) { return }

    return XML(json, { declaration: true, indent: "\t" })
}

const memoizedCreateXMLExample = memoizee(createXMLExample)

const memoizedSampleFromSchema = memoizee(sampleFromSchema)


const isImmutable = (maybe) => Im.Iterable.isIterable(maybe);

const getSampleSchema = (schema, contentType="", config={}) => {
    if (/xml/.test(contentType)) {
        if (!schema.xml || !schema.xml.name) {
            schema.xml = schema.xml || {}

            if (schema.$$ref) {
                let match = schema.$$ref.match(/\S*\/(\S+)$/)
                schema.xml.name = match[1]
            } else if (schema.type || schema.items || schema.properties || schema.additionalProperties) {
                return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!-- XML example cannot be generated -->"
            } else {
                return null
            }
        }
        return memoizedCreateXMLExample(schema, config)
    }

    return JSON.stringify(memoizedSampleFromSchema(schema, config), null, 2)
}


function fromJSOrdered (js) {
    if(isImmutable(js))
        return js // Can't do much here

    if (js instanceof window.File)
        return js

    return !isObject(js) ? js :
        Array.isArray(js) ?
            Im.Seq(js).map(fromJSOrdered).toList() :
            Im.OrderedMap(js).map(fromJSOrdered)
}

const getExampleComponent = ( sampleResponse, examples, HighlightCode, isOAS3 ) => {
    if ( examples && examples.size ) {
        return examples.entrySeq().map( ([ key, example ]) => {
            let exampleValue = example;
            let exampleSummary = null;
            if ( example.toJS ) {
                try {
                    const exampleJson = example.toJS();
                    exampleSummary = isOAS3 ? exampleJson.summary : null;
                    exampleValue = JSON.stringify( isOAS3 ? exampleJson.value : exampleJson, null, 2)
                }
                catch(e) {
                    exampleValue = String(example)
                }
            }

            return (<div key={ key }>
                <h5 style={{display: "inline-block"}}>{key}</h5>
                {isOAS3 && exampleSummary && <span style={{fontSize: "11px"}}>: {exampleSummary}</span>}
                <HighlightCode className="example" value={ exampleValue } />
            </div>)
        }).toArray()
    }

    if ( sampleResponse ) { return <div>
        <HighlightCode className="example" value={ sampleResponse } />
    </div>
    }
    return null
}

export default class Response extends React.Component {
    constructor(props, context) {
        super(props, context)

        this.state = {
            responseContentType: ""
        }
    }

    _onContentTypeChange = (value) => {
        const { onContentTypeChange, controlsAcceptHeader } = this.props
        this.setState({ responseContentType: value })
        onContentTypeChange({
            value: value,
            controlsAcceptHeader
        })
    }

    render() {
        let {
            code,
            response,
            className,
            specPath,
            fn,
            getComponent,
            getConfigs,
            specSelectors,
            contentType,
            controlsAcceptHeader
        } = this.props

        let { inferSchema } = fn
        let { isOAS3 } = specSelectors

        let headers = response.get("headers")
        let links = response.get("links")
        const Headers = getComponent("headers")
        const HighlightCode = getComponent("highlightCode")
        const ModelExample = getComponent("modelExample")
        const Markdown = getComponent( "Markdown" )
        const OperationLink = getComponent("operationLink")
        const ContentType = getComponent("contentType")

        var sampleResponse
        var schema, specPathWithPossibleSchema, examples

        if(isOAS3()) {
            const schemaPath = ["content", this.state.responseContentType, "schema"]
            const examplePath = ["content", this.state.responseContentType, "example"]
            const examplesPath = ["content", this.state.responseContentType, "examples"]
            examples = response.getIn(examplesPath)
            if(!examples) {
                const t = response.getIn(examplePath);
                if(t) examples = List().push(t)
            }
            const oas3SchemaForContentType = response.getIn(schemaPath)
            sampleResponse = oas3SchemaForContentType ? getSampleSchema(oas3SchemaForContentType.toJS(), this.state.responseContentType, {
                includeReadOnly: true
            }) : null;
            schema = oas3SchemaForContentType ? inferSchema(oas3SchemaForContentType.toJS()) : null
            specPathWithPossibleSchema = oas3SchemaForContentType ? schemaPath : specPath
        } else {
            examples = response.get("examples")
            schema = inferSchema(response.toJS()) // TODO: don't convert back and forth. Lets just stick with immutable for inferSchema
            specPathWithPossibleSchema = response.has("schema") ? [...specPath, "schema"] : specPath
            sampleResponse = schema ? getSampleSchema(schema, contentType, {
                includeReadOnly: true,
                includeWriteOnly: true // writeOnly has no filtering effect in swagger 2.0
            }) : null
        }
        if(examples) {
            examples = examples.map(example => {
                // Remove unwanted properties from examples
                return example.set ? example.set("$$ref", undefined) : example
            })
        }
        window.response = window.response || response;

        let example = getExampleComponent( sampleResponse, examples, HighlightCode, isOAS3() )
        window.examples = window.examples || examples;
        window.sampleResponse = window.sampleResponse || sampleResponse;

        window.schema = window.schema || schema;
        window.specPathWithPossibleSchema = window.specPathWithPossibleSchema || specPathWithPossibleSchema;

        return (
            <tr className={ "response " + ( className || "") }>
                <td className="col response-col_status">
                    { code }
                </td>
                <td className="col response-col_description">

                    <div className="response-col_description__inner">
                        <Markdown source={ response.get( "description" ) } />
                    </div>

                    { isOAS3 ?
                        <div className={cx("response-content-type", {
                            "controls-accept-header": controlsAcceptHeader
                        })}>
                            <ContentType
                                value={this.state.responseContentType}
                                contentTypes={ response.get("content") ? response.get("content").keySeq() : Seq() }
                                onChange={this._onContentTypeChange}
                            />
                            { controlsAcceptHeader ? <small>Controls <code>Accept</code> header.</small> : null }
                        </div>
                        : null }

                    { example ? (
                        <ModelExample
                            specPath={specPathWithPossibleSchema}
                            getComponent={ getComponent }
                            getConfigs={ getConfigs }
                            specSelectors={ specSelectors }
                            schema={ fromJSOrdered(schema) }
                            example={ example }/>
                    ) : null}

                    { headers ? (
                        <Headers
                            headers={ headers }
                            getComponent={ getComponent }
                        />
                    ) : null}


                </td>
                {specSelectors.isOAS3() ? <td className="col response-col_links">
                    { links ?
                        links.toSeq().map((link, key) => {
                            return <OperationLink key={key} name={key} link={ link } getComponent={getComponent}/>
                        })
                        : <i>No links</i>}
                </td> : null}
            </tr>
        )
    }
}