const _ = require('lodash');
const colors = require('colors');
const axios = require('axios');
const args = require('yargs')
    .demandOption(['clientId', 'clientSecret', 'domain'])
    .argv;
const fs = require('fs');
const Promise = require('bluebird');
const AWS = require('aws-sdk');
const { eulaText } = require('./eula-text');

axios.defaults.baseURL = `https://${args.domain}`;
AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));

const clientsToKeep = [
    "All Applications",
    "Management-NIC",
    /^Occm-Client/,
    "Users Dashboard","auth0-delegated-admin", //needed for the users admin extension
    "QStack-NIC", //temp clients for integration with Qstack
    "Test-SPA", //SPA client for testing purposes even in production
    "ReDash-SAML"
];
const apisToKeep = ["Auth0 Management API"];
const connectionsToKeep = [/^ldap-/, "NetAppTestWindows", "TestNetApp", "TestSamlConn", "NetAppSAML", "myAdfs", /^fed-/, /-waad/];
let clientArr;
let clientMap;
let notOccmClientArray;

const readAndFillFile = (fileName) => {
    const contents = fs.readFileSync(fileName, 'utf8');
    return contents
        .replace(/%hubspotSmtpUserName%/g, process.env.hubspotSmtpUserName)
        .replace(/%hubspotSmtpPassword%/g, process.env.hubspotSmtpPassword)
        .replace(/%REACT_APP_INTERCOM_APP_ID%/g, process.env.REACT_APP_INTERCOM_APP_ID)
        .replace(/%PORTAL_FRONTEND_BASE_URL%/g, process.env.PORTAL_FRONTEND_BASE_URL)
        .replace(/%PORTAL_BACKEND_BASE_URL%/g, process.env.PORTAL_BACKEND_BASE_URL)
        .replace(/%STAGE_DOT%/g, process.env.stage === 'prod' ? '' : process.env.stage + '.')
        .replace(/%OCCM_NAME%/g, "Cloud Volumes Ontap")
        .replace(/%OCCM_AUDIENCE%/g, "https://cloudmanager.cloud.netapp.com")
        .replace(/%GOVERNANCE_AUDIENCE%/g, "https://governance.cloud.netapp.com")
        .replace(/%REACT_APP_KUBERNETES_SERVICE_API_URL%/g, process.env.REACT_APP_KUBERNETES_SERVICE_API_URL)
        .replace(/%STAGE%/g, process.env.stage === 'prod' ? '' : process.env.stage);
};

const updateAccount = async () => {
    console.log(`STAGE: ${process.env.stage}`);

    axios.interceptors.response.use(undefined, (err) => {
        if (err.response.status === 429 && err.config && (!err.config.__retryCount || err.config.__retryCount < 3)) {
            err.config.__retryCount = err.config.__retryCount ? err.config.__retryCount + 1 : 1;
            return new Promise((resolve, reject) => {
                console.log("Call failed with 429 - retrying in 6000ms");
                setTimeout(() => {
                    axios(err.config).then(resolve, reject);
                }, 6000)
            });
        }
        throw err;
    });

    try {
        const {data: {access_token: accessToken}} = await axios.post("/oauth/token", {
            "grant_type": "client_credentials",
            "client_id": args.clientId,
            "client_secret": args.clientSecret,
            "audience": `https://${args.domain}/api/v2/`
        });
        axios.defaults.headers.common['Authorization'] = `Bearer ${accessToken}`;
    }
    catch(err){
        console.log(`Failed to retrieve oath token for domain: ${args.domain} with clientId: ${args.clientId}. Script exiting. No work done.`.red, err);
        process.exit(1);
    }

    try{
        await updateClients(); // must be updated first!
        await step(updateGlobalHostedLogin, "updating global hosted login page");
        await step(handleResource({
            directory: "apis",
            apiUrl: "/api/v2/resource-servers",
            itemsToKeep: apisToKeep,
            title: "API",
            idField: "id",
            ignorePropertiesFromUpdate: ["identifier"]
        }), "updating APIS");
        await step(handleResource({
            directory: "rules",
            apiUrl: "/api/v2/rules",
            itemsToKeep: [],
            title: "rules",
            idField: "id",
            ignorePropertiesFromUpdate: ["stage"]
        }), "updating Rules");
        await step(handleResource({
            directory: "connections",
            apiUrl: "/api/v2/connections",
            itemsToKeep: connectionsToKeep,
            title: "connections",
            idField: "id",
            ignorePropertiesFromUpdate: ["name"]
        }), "updating Connections");
        await step(updateEmailProvider, "updating email provider");
        await step(updateEmailTemplate("verify_email", "verification-email-template.html", "Please verify your email for your NetApp Cloud Account"), "updating verification email template");
        await step(updateEmailTemplate("reset_email", "change-password-email-template.html", "Password Change Request"), "updating reset password email template");
        await step(updateEmailTemplate("blocked_account", "blocked-account-email-template.html", "NetApp Cloud Account Blocked"), "updating blocked account template");
        await step(updateClientGrants, "updating client grants");
        await step(updateTenantSettings, "updating tenant settings");
        await step(updateClientsInDynamo, "updating static clients in dynamo");
    }
    catch (err) {
        console.log("Account update halted during one of the steps. Partial work may have been done.".red, err);
        process.exit(1);
    }
};


const step = async (fn, title)  => {
    console.log(`>   Starting ${title}`.grey);
    return fn().catch(err => {
        console.log(`$Failed {title}`.red, err);
        throw err;
    });
};

const handleResource = (resourceDescriptor) => async () => {
    if(!resourceDescriptor.directory) throw new Error("Resource descriptor missing directory");
    if(!resourceDescriptor.apiUrl) throw new Error("Resource descriptor missing apiUrl");
    if (!resourceDescriptor.title) throw new Error("Resource descriptor missing title");
    if(!resourceDescriptor.idField) throw new Error("Resource descriptor missing idField");
    if(!resourceDescriptor.itemsToKeep) throw new Error("Resource descriptor missing itemsToKeep");

    const dirPath = `src/${resourceDescriptor.directory}`;
    const dirContents = fs.readdirSync(dirPath).filter(name => {
        if(process.env.stage !== "dev" && name.startsWith("dev.")) return false;
        if(process.env.stage !== "staging" && name.startsWith("staging.")) return false;
        return name.endsWith(".json");
    });

    const specs = dirContents.map(file => {
        const path = `${dirPath}/${file}`;
        let json = null;
        try {
            json = JSON.parse(readAndFillFile(path));
        } catch (e) {
            console.log(`Unable to parse contents of ${path} as JSON.`.red, e);
            throw e;
        }

        if(!json.name){
            throw new Error(`${resourceDescriptor.title} json for ${path} does not contain a name`);
        }

        const replaceStage = (stage, obj, value, key) => {
            if(key.startsWith(`${stage}.`)){
                delete obj[key];
                if(process.env.stage === stage){
                    obj[key.substring(`${stage}.`.length)] = value;
                }
            }
        };

        const recurFill = (obj) => {
            _.forOwn(obj, (value, key) => {
                replaceStage("dev", obj, value, key);
                replaceStage("staging", obj, value, key);
                replaceStage("prod", obj, value, key);

                if(_.isPlainObject(value)) recurFill(value);
                else if(_.isString(value) && value.startsWith("<") && value.endsWith(">")){
                    console.log(`${dirPath}/${value.substring(1, value.length - 1)}`);
                    obj[key] = readAndFillFile(`${dirPath}/${value.substring(1, value.length - 1)}`);
                }
            });
        };

        recurFill(json);

        return json;
    });

    let existingItems;
    try {
        if (resourceDescriptor.existing) {
            existingItems = resourceDescriptor.existing;
        } else {
            const {data} = await axios.get(resourceDescriptor.apiUrl);
            existingItems = data;
        }
    } catch (err) {
        console.log(`Unable to retrieve list of existing ${resourceDescriptor.title}s`.red, err);
        throw err;
    }

    try {
        await deleteItems(existingItems, specs, resourceDescriptor);
        await createItems(existingItems, specs, resourceDescriptor);
        await updateItems(existingItems, specs, resourceDescriptor);
    } catch (err) {
        console.log(`Error deleting creating and updating ${resourceDescriptor.title}s`, err);
        throw err;
    }
};

const deleteItems = (existingItems, itemSpecs, resourceDescriptor) => {
    const itemNamesFromSpec = _.map(itemSpecs, "name");

    const itemsToRemove = existingItems.filter(({name}) => {
        return !itemNamesFromSpec.includes(name) && !resourceDescriptor.itemsToKeep.find(matcher => {
            if(matcher instanceof RegExp){
                return matcher.test(name)
            } else {
                return matcher === name;
            }
        })
    });

    console.log(`>>>   Attempting to delete the following ${resourceDescriptor.title}: ${_.map(itemsToRemove, "name")}`.grey);

    return Promise.all(itemsToRemove.map(async item => {
        await axios.delete(`${resourceDescriptor.apiUrl}/${item[resourceDescriptor.idField]}`);
        if (resourceDescriptor.existing) {
            _.remove(resourceDescriptor.existing, obj => obj.name === item.name);
            console.log(`Successfully deleted ${resourceDescriptor.title} '${item.name}'`.green);
        }
    })).then(() => {
        console.log(`Successfully deleted ${resourceDescriptor.title}s`.green);
    }, err => {
        console.log(`Failed to delete all ${resourceDescriptor.title}s`.red, err);
    });
};

const createItems = (existingItems, itemSpecs, resourceDescriptor) => {
    const existingItemNames = _.map(existingItems, "name");

    const itemsToCreate = itemSpecs.filter(({name}) => !existingItemNames.includes(name));

    console.log(`>>>   Attempting to create the following ${resourceDescriptor.title}s: ${_.map(itemsToCreate, "name")}`.grey);

    return Promise.all(itemsToCreate.map(async item => {
        const {data: createResult} = await axios.post(resourceDescriptor.apiUrl, item);
        if (resourceDescriptor.existing) {
            resourceDescriptor.existing.push({ name: createResult.name, client_id: createResult.client_id, displayName: _.get(item, 'client_metadata.displayName', null) });
            console.log(`Successfully created ${resourceDescriptor.title} '${item.name}'`.green);
        }
    }))
        .then(() => {
            console.log(`Successfully created ${resourceDescriptor.title}s`.green);
        }, err => {
            console.log(`Failed to create all ${resourceDescriptor.title}s`.red, err);
            throw err;
        });
};

const updateItems = (existingItems, itemSpecs, resourceDescriptor) => {
    const existingItemNames = _.map(existingItems, "name");

    const itemsToUpdate = itemSpecs.filter(({name}) => existingItemNames.includes(name));

    console.log(`>>>   Attempting to update the following ${resourceDescriptor.title}s: ${_.map(itemsToUpdate, "name")}`.grey);

    if (resourceDescriptor.existing) {
        _.forEach(itemsToUpdate, item => {
            if (_.get(item, 'client_metadata.displayName', null)) {
                const itemToUpdate = _.find(resourceDescriptor.existing, obj => obj.name === item.name);
                _.merge(itemToUpdate, { displayName: item.client_metadata.displayName });
            }
        })
    }

    return Promise.all(itemsToUpdate.map(itemSpec => {
        const id = _.find(existingItems, {name: itemSpec.name})[resourceDescriptor.idField];

        const itemSpecClone = _.cloneDeep(itemSpec);
        (resourceDescriptor.ignorePropertiesFromUpdate || []).forEach(ignore => {
            itemSpecClone[ignore] = undefined;
        });

        return axios.patch(`${resourceDescriptor.apiUrl}/${id}`, itemSpecClone)
    })).then(() => {
        console.log(`Successfully updated ${resourceDescriptor.title}s`.green);
    }, err => {
        console.log(`Failed to update all ${resourceDescriptor.title}s`.red, err);
        throw err;
    });
};

const updateClients = async () => {
    clientArr = await step(getClientArray, 'getting clients');
    await step(handleResource({
        directory: "clients",
        apiUrl: "/api/v2/clients",
        itemsToKeep: clientsToKeep,
        title: "client",
        idField: "client_id",
        existing: clientArr
    }), "updating clients");
    notOccmClientArray = _.filter(clientArr, client => !client.name.startsWith('Occm-Client'));
    clientMap = _.keyBy(notOccmClientArray, 'name');
};

const updateEmailProvider = () => {
    const emailProviderContent = readAndFillFile("src/email-provider.json");
    return axios.get("/api/v2/emails/provider").then(() => {
        return axios.patch;
    }, err => {
        if(err.response.status === 404) return axios.post;
        else throw err})
        .then(method => method("/api/v2/emails/provider", JSON.parse(emailProviderContent)))
        .then(() => {
            console.log("Successfully updated email provider".green);
        }, err => {
            console.log("Failed setting email provider".red, err);
            throw err;
        })   ;

};

const updateEmailTemplate = (templateName, templateSource, subject) => () => {
    let clientId = "";
    let clientSecret = "";

    /*
        These are the GLOBAL TENANT credentials.. used because update emails is api v1
        TODO: use v2 api
     */
    if(process.env.stage === "dev"){
        clientId = "r9iyZvWo6biKsSwA_M-Wseo4UUo7gZtX";
        clientSecret = "pjtKyzxXG9SYPoi7njCBTV26L6Q4jh3lpJk8Y2JPZElECOW-k7Jd70NItXHPRWAU";
    } else if (process.env.stage === "staging"){
        clientId = "2vh1KLNTef4x-OodrVUXzlZwP6A0N-rJ";
        clientSecret = "fHLzeH9C25FWTMlCU9NykQNjIMOaS2M6LIJlUGs3wYS-cEHIbvDDyyf2-W3fYosV";
    } else if (process.env.stage === "prod"){
        clientId = "D8HCLBWjgb4nTN_lT8YO83nV2lGgqus0";
        clientSecret = "SJpBRUYJqphS3SQhid2W0NVkMfcB0J1bzPFIBWGsQp7FDC-t5vBR1hiUBjQnd-gb";
    }

    return axios.post("/oauth/token", {
        "grant_type":"client_credentials",
        "client_id": clientId,
        "client_secret": clientSecret
    }).then(({data: {access_token: accessToken}}) => {
        return axios.get(`/api/emails/${templateName}`, {
            headers: {'Authorization': `Bearer ${accessToken}`}
        }).then(() => {
            return {method: axios.put, accessToken, url: `/api/emails/${templateName}`};
        }, err => {
            if(err.response.status === 404) return {method: axios.post, accessToken, url: "/api/emails"};
            else throw err;
        })
    }).then(({method, accessToken, url}) => {
        return method(url, {
            body: readAndFillFile(`src/email-templates/${templateSource}`),
            disabled: false,
            from: "saas.support@netapp.com",
            resultUrl: "{{application.callback_domain}}",
            subject: subject,
            syntax: "liquid",
            template: templateName,
            urlLifetimeInSeconds: 432000
        }, {
            headers: {'Authorization': `Bearer ${accessToken}`}
        })
    }).then(() => {
        console.log(`Successfully updated ${templateName} template`.green);
    }, err => {
        console.log(`Failed setting ${templateName} template`.red, err);
        throw err;
    });
};

const updateClientGrants = async () => {
    const clientGrants = JSON.parse(readAndFillFile("src/client-grants.json"));

    return Promise.all(clientGrants.map(({name, scope, audience}) => {
        const clientId = clientMap[name].client_id;
        console.log(`Updating grant for ${name} with id: ${clientId} and audience: ${audience}`);

        return axios.get(`/api/v2/client-grants?client_id=${clientId}`)
            .then(({data: clientGrants}) => {
                const grantWithAudience = clientGrants.find(grant => grant.audience === audience);

                if (!grantWithAudience) {
                    console.log(`No existing client grants for client with given audience, will create`);
                    return axios.post("/api/v2/client-grants", {
                        client_id: clientId,
                        audience,
                        scope
                    })
                } else {
                    if (grantWithAudience && _.isEqual(grantWithAudience.scope, scope)) {
                        console.log("Scopes are identical, no update required.");
                    } else  {
                        console.log("Scopes are different, attempting update.");
                        return axios.patch(`/api/v2/client-grants/${grantWithAudience.id}`, {
                            scope: scope
                        })
                    }
                }

                console.log("Successfully client grants".green);
            }, err => {
                console.log("Unable to update client grants".red, err);
                throw err;
            })
    }));
};


const updateGlobalHostedLogin = () => {
    const loginPageClients = _.filter(_.map(notOccmClientArray, obj => { return {client_id: obj.client_id, displayName: obj.displayName }}), obj => obj.displayName && obj.displayName !== 'NetApp Cloud Central') ;

    const loginPageContent = readAndFillFile("src/HostedPage.html")
        .replace(/%CLIENT_IDS_AND_DISPLAY_NAME_LIST%/, JSON.stringify(loginPageClients))
        .replace(/%EULA_TEXT%/, eulaText);

    const allAppsClientId = clientMap["All Applications"].client_id;
    console.log(`All Applications Client Id is: ${allAppsClientId}`);

    return axios.patch(`/api/v2/clients/${allAppsClientId}`, {
        custom_login_page_on: true,
        custom_login_page: loginPageContent
    })
        .then(() => {
            console.log("Successfully updated global hosted login page".green);
        }, err => {
            console.log("Unable to update global hosted login page".red, err);
            throw err;
        })
};

const updateClientsInDynamo = () => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();

    console.log("Static client list:");
    console.log(notOccmClientArray);

    return Promise.each(notOccmClientArray, client => {
        const updateParams = {
            TableName: process.env.staticClientTable,
            Key: { clientId: client.client_id },
            AttributeUpdates: {
                clientName: { Action: "PUT", Value: client.name }
            }
        };

        if (client.displayName){
            updateParams.AttributeUpdates.displayName = { Action: "PUT", Value: client.displayName }
        }

        return dynamoDb.update(updateParams).promise();
    }).then(() => {
        console.log("Successfully updated static clients in dynamo".green);
    }, err => {
        console.log("Failed updating static clients in dynamo".red, err);
        throw err;
    });
};

const updateTenantSettings = () => {
    const tenantSettings = readAndFillFile("src/tenant-settings.json");
    const hostedPasswordResetPage = readAndFillFile("src/HostedPasswordResetPage.html");

    return axios.patch("/api/v2/tenants/settings", Object.assign(JSON.parse(tenantSettings), {change_password: {enabled: true, html: hostedPasswordResetPage}}))
        .then(() => {
            console.log("Successfully updated tenant settings".green);
        }, err => {
            console.log("Failed setting tenant settings".red, err);
            throw err;
        })
};

const getClientArray = (delay = 0) => {
    console.log(`Attempting to get all auth0 clients.`);
    return getAllClientsRecursive([], 0, delay)
        .then(clientArr => {
            const nonOccmClients = _.filter(clientArr, client => !client.name.startsWith('Occm-Client'));
            const nonOccmLength = nonOccmClients.length;
            const uniqNonOccmClientsLength = _.uniqBy(nonOccmClients, 'name').length;
            if(nonOccmLength !== uniqNonOccmClientsLength) {
                throw new Error(`Duplicate (non OCCM) clients found! Exiting.`);
            }

            console.log(`Success. Got all ${clientArr.length} clients.`);
            return clientArr;
        }, err => {
            console.error(`Error. getting all clients failed`, err);
            throw err;
        })
};

const getAllClientsRecursive = (acc, lastPage, delay) => {
    const getAllClients = (page) => {
        return Promise.delay(delay).then(() => {
            console.log(`Getting auth0 clients ${page*100}-${(page+1)*100}...`);
            let paramsObject = {
                per_page: 100,
                page: page,
                include_totals: true,
                search_engine: 'v2',
                fields: 'client_id,name'
            };
            return axios.get('api/v2/clients', {
                params: paramsObject
            })
        })
    };


    const shouldDoAgain = (data) => data.start + data.limit < data.total;
    return getAllClients(lastPage)
        .then(({data}) => {
            const newAcc = acc.concat(data.clients);
            if (shouldDoAgain(data)) {
                return getAllClientsRecursive(newAcc, lastPage + 1, delay);
            } else {
                return newAcc;
            }
        })
};

updateAccount();