const _ = require("lodash");
const Joi = require('joi');
const HubSpot = require('hubspot');
const Promise = require('bluebird');
const axios = require('axios');

const dispatch = require('./dispatch');
const { redirectPromoLink } = require('./promo-apis.js');
const { ForbiddenError } = require("./errors");

const hubspotClient = new HubSpot({ apiKey: process.env.hubspotAPIKey }); // "19292cdf-b30d-4b13-92b4-3e080670d32c"
const superSecretAnalyticsKey = "b5c35b09-552b-4b30-bbbc-1b07a7971d27";

const validateAnalyticsKey = key => key === superSecretAnalyticsKey;

const splitName = exports.splitName = fullName => {
    const nameArr = fullName.split(/\s+/);
    if (nameArr.length === 1) { return { firstname: nameArr[0], lastname: "" }; }

    const firstname = nameArr.slice(0, -1).join(" ");
    const lastname = nameArr.pop();
    return { firstname, lastname };
};

const getService = clientName => {
    let service = 'Portal';
    if (clientName.startsWith("Occm-Client")) { service = 'ONTAP Cloud'; }
    if (clientName.startsWith("Cloud-Sync")) { service = 'Cloud Sync'; }
    return service;
};

const isGarbageUser = email => {
    let isGarbage = false;
    if (email.endsWith("@netapp.com")) { return true; }
    const QAUsers = ['occmuser', 'dfio_', 'stagoccm', 'occmmultiregion', 'occmfakeuser', 'trialexpiryfakeuser'];
    QAUsers.forEach(user => {
        if (email.includes(user)) { isGarbage = true; }
    });
    return isGarbage;
};

const createHubspotProperties = exports.createHubspotProperties = inputJson =>
    _.map(inputJson, (val, key) => ({ property: key, value: val }));

const createOrUpdateRetry = (email, properties, tries) => hubspotClient.contacts.createOrUpdate(email, properties)
    .then(data => {
        console.log(`Successfully ${data.isNew ? "CREATED" : "UPDATED"} hubspot user for user: ${email}, hubspot vid is ${data.vid}`);
    })
    .catch(err => {
        if (err.statusCode === 429) {
            if (tries <= 3) {
                console.log(`Reached secondly limit on attempt ${tries}, retrying update call for user ${email}...`);
                return Promise.delay(2500).then(() => createOrUpdateRetry(email, properties, tries + 1));
            }
            console.log(`Reached retry limit. Update call failed for user: ${email}`);
            console.log(err);
            throw err;
        }
        console.log(`createOrUpdate user failed for user: ${email} error:`);
        console.log(err);
        throw err;
    });

const userLoginSignUp = exports.userLoginSignUp = ({
    user, context, eventType, analyticsKey
}) => {
    console.log(`Event: ${eventType} for user: ${user.email}. Attempting to updateOrCreate hubspot user.`);

    if (!validateAnalyticsKey(analyticsKey)) { return Promise.reject(new ForbiddenError()); }
    if (isGarbageUser(user.email)) {
        console.log(`Garbage email. Not creating hubspot user for email ${user.email}`);
        return Promise.resolve();
    }
    if (context.clientName === 'Widget-SPA') {
        console.log(`Widget login, not sending update to hubspot`);
        return Promise.resolve();
    }

    const nameObject = splitName(_.get(user, "user_metadata.name", ""));
    const propertyJson = {
        email: user.email,
        firstname: nameObject.firstname,
        lastname: nameObject.lastname,
        company: _.get(user, "user_metadata.company", ""),
        auth0_user_id: user.user_id,
        portal_email_validated: user.email_verified ? "Yes" : "No",
        portal_last_login_date: new Date().getTime(),
        portal_number_of_logins: _.get(context, "stats.loginsCount", -1)
    };

    if (eventType === 'sign-up') {
        propertyJson.portal_signups_origin = getService(context.clientName);
        propertyJson.portal_signup_date = new Date(user.created_at).getTime();
    }

    const properties = { properties: createHubspotProperties(propertyJson) };

    return createOrUpdateRetry(user.email, properties, 1);
};

const cloudAssessmentFlowLabelsForHubspot = ["redirectToLoginFromHubspot", "/", "azure", "aws", "azureStarted", "azureSuccess", "awsStarted", "awsSuccessApi", "awsErrorApi", "azureError"];
const campaignLabelsForHubSpot = ["cvoFreeTrial"];
const allowedHubSpotLabels = cloudAssessmentFlowLabelsForHubspot.concat(campaignLabelsForHubSpot);

const handleEvent = ({
    category, action, label, userId, userEmail, extra
}) => {
    console.log(`Received analytics event handle for user: ${userId || 'anon'} with email: ${userEmail || 'anon'}. Category-Action-Label: ${category}-${action}-${label}, extra: ${JSON.stringify(extra)}`);

    if ((category === "CloudAssessment" || category === "SpecialCampaign")
        && (process.env.stage === "prod" || process.env.forceTrackToHubspot === 'true')
        && (allowedHubSpotLabels.includes(label) || action === "ExternalLinkClick")) {
        const eventName = `${category}-${action}-${label}`;
        const email = encodeURIComponent(userEmail);
        const url = `https://track.hubspot.com/v1/event?_n=${eventName}&_a=525875&email=${email}`;
        let params = {};

        if (label === "awsSuccessApi") {
            const result = _.get(extra, "allRegions[0].result");
            params = {
                aws_assessment_ebs_savings: _.get(result, "efficiency.unusedVolumesCost"),
                aws_assessment_s3_savings: _.get(result, "efficiency.unusedSnapshotsCost"),
                aws_assessment_capacity_gb_: _.get(result, "totalCapacity"),
                aws_assessment_avg_rpo: _.get(result, "protection.averageRPO"),
                aws_assessment_avg_snapshots_per_volume: _.get(result, "protection.averageNumberOfSnapshots")
            };
        } else if (label === "azureSuccess") {
            const result = _.get(extra, "result");
            params = {
                azure_assessment_disks_savings: _.get(result, "efficiency.unattachedDisksAnnualCost"),
                azure_assessment_snapshots_savings: _.get(result, "efficiency.unassociatedSnapshotsAnnualCost"),
                azure_assessment_capacity: _.get(result, "totalCapacity"),
                azure_assessment_avg_rpo: _.get(result, "protection.averageRPO"),
                azure_assessment_avg_snapshots_per_disk: _.get(result, "protection.averageNumberOfSnapshots")
            };
        } else if (label === "awsErrorApi") {
            const message = _.get(extra, "message");

            if (message.includes('The provided AWS credentials are invalid')) {
                params = { aws_assessment_failure_reason: 'Invalid AWS Credentials' };
            } else if (message.includes('Request limit exceeded')) {
                params = { aws_assessment_failure_reason: 'Request Limit Exceeded' };
            } else if (message.includes('The provided credentials do not have the required permission')) {
                params = { aws_assessment_failure_reason: 'Invalid Access Level' };
            } else if (message.includes('You are not subscribed to this service')) {
                params = { aws_assessment_failure_reason: 'Not Subscribed to Service' };
            } else if (message.includes('Network Failure')) {
                params = { aws_assessment_failure_reason: 'Network Failure' };
            }
        } else if (label === "cvoFreeTrial") {
            params = {
                recent_campaign: "ActiveIQ CVO – DR Q4FY19",
                recent_channel: "Email",
                recent_lead_source: "Direct"
            };
        }

        console.log(`sending to HubSpot ${url} with params ${JSON.stringify(params)}`);

        return axios.get(url, { params }).then(response => {
            console.log(`Successfully reported to url: ${response.request._redirectable._currentUrl}`);
        }, err => {
            console.log(err);
            return null;
        });
    }
    return Promise.resolve();
};

const routes = [
    {
        method: "POST",
        pattern: "/analytics/auth0",
        bodySchema: Joi.object().keys({
            user: Joi.object().required(),
            context: Joi.object().required(),
            eventType: Joi.string().valid('login', 'sign-up').required(),
            analyticsKey: Joi.string().required()
        }),
        handler: ({ body }) => userLoginSignUp({
            user: body.user, context: body.context, eventType: body.eventType, analyticsKey: body.analyticsKey
        })
    },
    {
        method: "GET",
        pattern: "/analytics/promo/:promoId",
        handler: ({ urlParams, queryParams }) => redirectPromoLink({ promoId: urlParams.promoId, track: queryParams && queryParams.track }),
        audit: {
            include: true
        }
    },
    {
        method: "POST",
        pattern: "/analytics/event",
        bodySchema: Joi.object().keys({
            category: Joi.string().required(),
            action: Joi.string().required(),
            label: Joi.string().optional(),
            userId: Joi.string(),
            userEmail: Joi.string(),
            extra: Joi.object()

        }).or('userId', 'userEmail', 'extra'),
        handler: ({ body }) => handleEvent({ ...body })
    }
];

exports.handler = (event, context, callback) => {
    dispatch({
        event, callback, routes, context
    });
};
