const jwt = require('jsonwebtoken');
const jwksClient = require('jwks-rsa');

const client = jwksClient({
    jwksUri: `https://${process.env.managementDomain}/.well-known/jwks.json`
});

const extractAuthInfo = exports.extractAuthInfo = methodArn => {
    const tmp = methodArn.split(':');
    const apiGatewayArnTmp = tmp[5].split('/');

    return {
        accountId: tmp[4],
        region: tmp[3],
        restApiId: apiGatewayArnTmp[0],
        stage: apiGatewayArnTmp[1],
        method: apiGatewayArnTmp[2]
    };
};

const generatePolicy = exports.generatePolicy = (payload, effect, methodArn) => {
    const authInfo = extractAuthInfo(methodArn);

    return {
        principalId: payload.sub,
        context: {
            scope: payload.scope,
            fullName: payload['http://cloud.netapp.com/full_name'],
            internal: payload['http://cloud.netapp.com/internal']
        },
        policyDocument: {
            Version: '2012-10-17',
            Statement: [{
                Action: 'execute-api:Invoke',
                Effect: effect,
                Resource: `arn:aws:execute-api:${authInfo.region}:${authInfo.accountId}:${authInfo.restApiId}/${authInfo.stage}/*/*`
            }]
        }
    };
};

const validateAndExtractToken = exports.validateAndExtractToken = (token, audience, method) => new Promise((resolve, reject) => {
    try {
        console.log(token);
        const decoded = jwt.decode(token, { complete: true });
        console.log(decoded);

        if (!decoded.header || !decoded.header.kid) {
            reject(new Error("Unauthorized"));
        } else {
            client.getSigningKey(decoded.header.kid, (err, key) => {
                if (!key) {
                    console.log("Failed to resolve signing key..");
                    err.method = method;
                    console.log(err);
                    reject(new Error("Unauthorized"));
                } else {
                    const signingKey = key.publicKey || key.rsaPublicKey;
                    jwt.verify(token, signingKey, {
                        audience,
                        issuer: `https://${process.env.managementDomain}/`,
                        algorithms: ['RS256']
                    }, (err, payload) => { // eslint-disable-line no-shadow
                        if (err) {
                            console.log("Failed to verify auth token");
                            err.method = method;
                            console.log(err);
                            reject(new Error("Unauthorized"));
                        } else {
                            try {
                                console.log("Successfully authorized");
                                resolve(payload);
                            } catch (e) {
                                console.err("Unable to authorize", e);
                                reject(new Error(e.message));
                            }
                        }
                    });
                }
            });
        }
    } catch (e) {
        console.log(e);
        reject(new Error("Unauthorized"));
    }
});


const authorizerTemplate = exports.authorizerTemplate = (event, context, callback, audience) => {
    console.log(event);

    let method = "";
    try {
        [method] = event.methodArn.match("/.*");
    } catch (e) {
        method = event.methodArn;
    }

    if (!event.authorizationToken || !event.authorizationToken.startsWith("Bearer ")) {
        callback("Unauthorized");
    } else {
        const token = event.authorizationToken.substring("Bearer ".length);
        validateAndExtractToken(token, audience, method)
            .then(
                payload => callback(null, generatePolicy(payload, 'Allow', event.methodArn)),
                err => callback(err)
            );
    }
};


exports.handler = (event, context, callback) => {
    authorizerTemplate(event, context, callback, "https://api.cloud.netapp.com");
};
