const UrlPattern = require('url-pattern');
const Joi = require('joi');
const _ = require("lodash");

const { sendToKinesisLogStream } = require("./utils/logging");
const {
    ClientError, ResourceNotFoundError, ForbiddenError, MethodNotAllowedError, mapsErrors
} = require("./errors");

const createAuditContext = ({ route, requestId, event }) => {
    if (process.env.auditAll !== 'true' && ((route.method === "GET" && !_.get(route, "audit.include")) || _.get(route, "audit.exclude"))) {
        return null;
    }
    return {
        internalUserFlag: _.get(event, "requestContext.authorizer.internal", undefined),
        pattern: route.pattern,
        method: route.method,
        requestId,
        referer: _.get(event, "headers.Referer"),
        sourceIp: _.get(event, 'requestContext.identity.sourceIp')
    };
};

const auditOpen = (auditContext, data) => {
    if (!auditContext || !process.env.kinesisLogStream) {
        return Promise.resolve();
    }
    const kinesisInput = {
        type: "AUDIT-OPEN",
        timestamp: new Date().valueOf(),
        auditContext,
        data
    };

    return sendToKinesisLogStream({ data: kinesisInput, key: auditContext.requestId }).then(() => {
        auditContext.requestData = data;
    }).catch(err => {
        auditContext.requestData = data;
        console.log(`Failed opening audit record for ${auditContext.requestId}`);
        console.log(err);
        return null;
    });
};

const auditClose = (auditContext, data) => {
    if (!auditContext || !process.env.kinesisLogStream) {
        return Promise.resolve();
    }
    const kinesisInput = {
        type: "AUDIT-CLOSE",
        timestamp: new Date().valueOf(),
        auditContext,
        data: Object.assign({}, data, { status: data.statusCode >= 400 ? "FAILED" : "SUCCESS" })
    };

    return sendToKinesisLogStream({ data: kinesisInput, key: auditContext.requestId }).catch(err => {
        console.log(`Failed closing audit record for ${auditContext.requestId}`);
        console.log(err);
        return null;
    });
};

module.exports = ({
    event, callback, routes, context
}) => {
    const principalId = _.get(event, "requestContext.authorizer.principalId", undefined);
    const method = event.httpMethod;
    const { path } = event;
    const scopesAll = _.get(event, "requestContext.authorizer.scope", undefined);
    const scopes = scopesAll ? scopesAll.split(" ") : undefined;
    const queryParams = _.get(event, "queryStringParameters", undefined);
    console.log(`Method: ${event.httpMethod}, Path: ${event.path}, principalId: ${principalId}, scopes: ${scopesAll}`);

    try {
        let auditContext = null;

        const respond = (statusCode, body) => {
            console.log(`Response ${statusCode}`);
            console.log(body);

            auditClose(auditContext, { statusCode, body }).then(() => {
                callback(null, {
                    isBase64Encoded: false,
                    statusCode,
                    headers: {
                        "Access-Control-Allow-Origin": "*"
                    },
                    body: body ? JSON.stringify(body) : null
                });
            });
        };

        const redirect = (statusCode, link) => {
            console.log(`Redirect response ${statusCode} link: ${link}`);

            auditClose(auditContext, { statusCode, link }).then(() => {
                callback(null, {
                    isBase64Encoded: false,
                    statusCode,
                    headers: {
                        "Access-Control-Allow-Origin": "*",
                        Location: link
                    },
                    body: null
                });
            });
        };

        const dispatch = () => {
            let body = null;
            try {
                if (event.body) { body = JSON.parse(event.body); } else { body = null; }
            } catch (e) {
                return Promise.reject(new ClientError("Unable to parse message body as json."));
            }

            if (body) {
                console.log("Request body:");
                console.log(body);
            }

            let matchedPattern = false;
            let urlParams = null;
            const route = routes.find(el => {
                const match = new UrlPattern(el.pattern).match(path);
                urlParams = match;
                matchedPattern = matchedPattern || !!match;
                return match && el.method === method;
            });

            if (route) {
                auditContext = createAuditContext({ route, requestId: context.awsRequestId, event });
                return auditOpen(auditContext, {
                    body, queryParams, principalId, urlParams, scopes
                }).then(() => {
                    if (route.scopes && _.difference(route.scopes, scopes).length > 0) {
                        console.log(`Route required scopes: ${route.scopes}`);
                        return Promise.reject(new ForbiddenError());
                    }

                    if (route.bodySchema) {
                        if (!route.bodyOptional || (route.bodyOptional && body)) {
                            const validation = Joi.validate(body, route.bodySchema, { allowUnknown: true, convert: false });
                            if (validation.error) {
                                return Promise.reject(new ClientError(`Body failed validation. ${validation.error.details.map(detail => detail.message).join(", ")}`));
                            }
                        }
                    }

                    if (route.querySchema) {
                        const validation = Joi.validate(queryParams, route.querySchema, { allowUnknown: true, convert: false });
                        if (validation.error) {
                            return Promise.reject(new ClientError(`Query Params failed validation. ${validation.error.details.map(detail => detail.message).join(", ")}`));
                        }
                    }

                    return route.handler({
                        principalId, body, queryParams, urlParams
                    });
                });
            }
            if (matchedPattern) {
                return Promise.reject(new MethodNotAllowedError());
            }
            return Promise.reject(new ResourceNotFoundError());
        };

        return dispatch().then(response => {
            if (response && response.statusCode) {
                redirect(response.statusCode, response.link);
            } else {
                const statusCode = response ? 200 : 204;
                respond(statusCode, response);
            }
        }, err => {
            const mapped = mapsErrors(err);
            if (mapped.statusCode === 500) {
                console.log("Dispatch failed with unhandled error:");
                console.log(err);
            }
            respond(mapped.statusCode, mapped.response);
        });
    } catch (e) {
        console.log("Failed to execute dispatch!!! CRITICAL!!!");
        console.log(e);
        return Promise.resolve(null);
    }
};
