const Joi = require('joi');
const _ = require("lodash");

const dispatch = require("./dispatch");

const mockEvent = arg => ({
    httpMethod: "POST",
    path: "/test-path",
    body: _.get(arg, "body"),
    requestContext: {
        authorizer: {
            principalId: "user-1",
            scope: ""
        }
    }
});

const successfulHandler = () => Promise.resolve();

const context = { awsRequestId: "1234" };

describe("Dispatcher", () => {
    it("should return 404 if no routes match", () => {
        const callback = jest.fn();
        const promise = dispatch({
            event: mockEvent(), callback, routes: [], context
        });

        return promise.then(() => {
            expect(callback.mock.calls[0][1].statusCode).toEqual(404);
        });
    });

    it("should return 405 if url matches but not method", () => {
        const callback = jest.fn();
        const promise = dispatch({
            event: mockEvent(),
            context,
            callback,
            routes: [{
                method: "GET",
                pattern: "/test-path",
                handler: successfulHandler
            }]
        });

        return promise.then(() => {
            expect(callback.mock.calls[0][1].statusCode).toEqual(405);
        });
    });

    it("should return 403 if not relevant scope", () => {
        const callback = jest.fn();
        const promise = dispatch({
            event: mockEvent(),
            context,
            callback,
            routes: [{
                method: "POST",
                scopes: ["need-this"],
                pattern: "/test-path",
                handler: successfulHandler
            }]
        });

        return promise.then(() => {
            expect(callback.mock.calls[0][1].statusCode).toEqual(403);
        });
    });

    it("should return 400 if missing required elements from body", () => {
        const callback = jest.fn();
        const promise = dispatch({
            event: mockEvent({ body: JSON.stringify({}) }),
            callback,
            context,
            routes: [{
                method: "POST",
                pattern: "/test-path",
                bodySchema: Joi.object().keys({
                    required: Joi.string().required()
                }),
                handler: successfulHandler
            }]
        });

        return promise.then(() => {
            expect(callback.mock.calls[0][1].statusCode).toEqual(400);
            expect(JSON.parse(callback.mock.calls[0][1].body).message).toEqual("Body failed validation. \"required\" is required");
        });
    });

    it("should return 200 and marshall properly if there is a body", () => {
        const callback = jest.fn();
        const promise = dispatch({
            event: mockEvent(),
            callback,
            context,
            routes: [{
                method: "POST",
                pattern: "/test-path",
                handler: () => Promise.resolve({ key: "value" })
            }]
        });

        return promise.then(() => {
            expect(callback.mock.calls[0][1].statusCode).toEqual(200);
            expect(JSON.parse(callback.mock.calls[0][1].body)).toEqual({ key: "value" });
        });
    });

    it("should return 204 if there is no body", () => {
        const callback = jest.fn();
        const promise = dispatch({
            event: mockEvent(),
            callback,
            context,
            routes: [{
                method: "POST",
                pattern: "/test-path",
                handler: successfulHandler
            }]
        });

        return promise.then(() => {
            expect(callback.mock.calls[0][1].statusCode).toEqual(204);
            expect(JSON.parse(callback.mock.calls[0][1].body)).toEqual(null);
        });
    });

    it("should return 500 in case of an unexpected error", () => {
        const callback = jest.fn();
        const promise = dispatch({
            event: mockEvent(),
            callback,
            context,
            routes: [{
                method: "POST",
                pattern: "/test-path",
                handler: () => Promise.reject(new Error("Some unexpected error"))
            }]
        });

        return promise.then(() => {
            expect(callback.mock.calls[0][1].statusCode).toEqual(500);
            expect(JSON.parse(callback.mock.calls[0][1].body).message).toEqual("Some unexpected error");
        });
    });

    it("should return 400 if the given body is not json", () => {
        const callback = jest.fn();
        const promise = dispatch({
            event: mockEvent({ body: "not json" }),
            callback,
            context,
            routes: [{
                method: "POST",
                pattern: "/test-path",
                handler: successfulHandler
            }]
        });

        return promise.then(() => {
            expect(callback.mock.calls[0][1].statusCode).toEqual(400);
            expect(JSON.parse(callback.mock.calls[0][1].body).message).toEqual("Unable to parse message body as json.");
        });
    });
});
