const AWS = require('aws-sdk');
const _ = require('lodash');

const { authAxios } = require("./../auth-utils");
const { ForbiddenError, ClientError } = require("./../errors");

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));

const getClientPermissionList = async ({ userId }) => {
    console.log(`Getting client permission list`);

    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const userInfoParams = {
        TableName: process.env.userInfoTable,
        Key: { userId }
    };

    const userInfoResponse = await dynamoDb.get(userInfoParams).promise();
    const allowedClients = _.get(userInfoResponse, "Item.developmentClients.values", []);
    const allowAllClients = allowedClients.includes("*");

    return { allowedClients, allowAllClients };
};

const fetchDetailsForClient = async ({ clientId }) => {
    console.log(`Fetching details for client ${clientId}`);
    const authClient = await authAxios();
    const clientResponse = (await authClient.get(`clients/${clientId}`)).data;
    console.log("Details successfully retrieved for client");

    return {
        name: clientResponse.name,
        domain: `${clientResponse.tenant}.auth0.com`,
        clientId: clientResponse.client_id,
        displayName: (clientResponse.client_metadata || {}).displayName,
        allowedCallbackUrls: clientResponse.callbacks,
        allowedWebOrigins: clientResponse.web_origins,
        allowedLogoutUrls: clientResponse.allowed_logout_urls,
        allowedCors: clientResponse.allowed_origins
    };
};

const updateClientDetails = async ({
    clientId, body: {
        allowedCallbackUrls, allowedWebOrigins, allowedLogoutUrls, allowedCors
    }
}) => {
    console.log(`Updating details for client ${clientId}`);

    const authClient = await authAxios();
    try {
        const clientResponse = (await authClient.patch(`clients/${clientId}`, {
            callbacks: allowedCallbackUrls,
            allowed_origins: allowedCors,
            web_origins: allowedWebOrigins,
            allowed_logout_urls: allowedLogoutUrls
        })).data;
        console.log("Details successfully retrieved for client");

        return {
            name: clientResponse.name,
            domain: `${clientResponse.tenant}.auth0.com`,
            clientId: clientResponse.client_id,
            displayName: (clientResponse.client_metadata || {}).displayName,
            allowedCallbackUrls: clientResponse.callbacks,
            allowedWebOrigins: clientResponse.web_origins,
            allowedLogoutUrls: clientResponse.allowed_logout_urls,
            allowedCors: clientResponse.allowed_origins
        };
    } catch (err) {
        const statusCode = _.get(err, 'response.status', null);
        const message = _.get(err, 'response.data.message', "");
        if (statusCode === 400 && message.includes('Payload validation error')) {
            return Promise.reject(new ClientError(message));
        }
        return Promise.reject(err);
    }
};

const getClients = exports.getClients = async ({ userId }) => {
    console.log(`Fetching clients for user ${userId}`);

    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const { allowedClients, allowAllClients } = await getClientPermissionList({ userId });

    console.log(`Clients permitted on user: ${allowedClients}`);
    console.log(`Fetching static clients permitted on user`);

    let response = [];
    if (allowAllClients) {
        const staticClientParams = {
            TableName: process.env.staticClientTable
        };
        response = (await dynamoDb.scan(staticClientParams).promise()).Items;
    } else if (allowedClients.length > 0) {
        const staticClientParams = {
            RequestItems: {
                [process.env.staticClientTable]: {
                    Keys: allowedClients.map(clientId => ({ clientId }))
                }
            }
        };

        response = (await dynamoDb.batchGet(staticClientParams).promise()).Responses[process.env.staticClientTable];
    }

    const clients = (response || []).map(client => ({
        name: client.clientName,
        clientId: client.clientId
    }));

    let selectedClient = null;

    if (clients.length > 0) {
        console.log(`${clients.length} static clients permitted on user. Fetching details for first client`);
        selectedClient = await fetchDetailsForClient({ clientId: clients[0].clientId });
    } else {
        console.log(`No static clients permitted on user`);
    }

    return Promise.resolve({
        clients,
        selectedClient
    });
};

const getClient = exports.getClient = async ({ userId, clientId }) => {
    console.log(`Getting client ${clientId} for user ${userId}`);

    const { allowedClients, allowAllClients } = await getClientPermissionList({ userId });

    if (allowAllClients || allowedClients.includes(clientId)) {
        return fetchDetailsForClient({ clientId });
    }
    console.log("User is not allowed to view details for client");
    return Promise.reject(new ForbiddenError());
};

exports.patchClient = async ({ clientId, userId, body }) => {
    console.log(`Updating client ${clientId}, for user ${userId}`);

    const { allowedClients, allowAllClients } = await getClientPermissionList({ userId });

    if (allowAllClients || allowedClients.includes(clientId)) {
        return updateClientDetails({ clientId, body });
    }

    console.log("User is not allowed to edit details for client");
    return Promise.reject(new ForbiddenError());
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("get-clients", "get-clients", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        getClients({ userId: argv.userId }).then(r => console.log(r), e => console.log(e));
    })
    .command("get-client", "get-client", yargs => {
        yargs.demandOption(['userId', 'clientId']);
    }, argv => {
        getClient({ userId: argv.userId, clientId: argv.clientId }).then(r => console.log(r), e => console.log(e));
    }).argv;
