const _ = require("lodash");
const AWS = require('aws-sdk');

const { getUserByEmail } = require('./../common');
const { authAxios } = require("./../auth-utils");
const { ClientError, ForbiddenError } = require("./../errors");

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));

const getSubordinateUsers = async ({ userId }) => {
    console.log(`Getting list of subordinate users for userId ${userId}.`);

    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const userInfoParams = {
        TableName: process.env.userInfoTable,
        Key: { userId }
    };

    const userInfoResponse = await dynamoDb.get(userInfoParams).promise();
    const subordinateUsers = _.get(userInfoResponse, "Item.subordinateUsers.values", []);

    console.log(`Successfully found ${subordinateUsers.length} subordinate users.`);

    return subordinateUsers;
};

const resetSubordinatePassword = exports.resetSubordinatePassword = async ({ userId, subordinateEmail, subordinateNewPassword }) => {
    const authClient = await authAxios();

    // 1. Get subordinate user by email
    const subordinateUser = await getUserByEmail({ email: subordinateEmail });

    if (!subordinateUser) {
        throw new ClientError(`Could not find user with Email ${subordinateEmail}.`);
    }
    const subordinateUserId = subordinateUser.user_id;

    // 2. Get list of subordinate users for the calling user from dynamo
    const subordinateUsers = await getSubordinateUsers({ userId });

    // 3. Ensure the calling user is really superordinate of the subordinate user
    if (!_.includes(subordinateUsers, subordinateUserId)) {
        throw new ForbiddenError(`User ${userId} is not allowed to reset password for user with Email ${subordinateEmail}.`);
    }

    // 4. Do update user with new password
    await authClient.patch(`users/${subordinateUserId}`, { password: subordinateNewPassword })
        .catch(err => {
            const statusCode = _.get(err, "response.data.statusCode");
            const errMessage = _.get(err, "response.data.message");
            if (statusCode === 400 && errMessage === 'PasswordStrengthError: Password is too weak') {
                throw new ClientError(errMessage);
            } else {
                throw err;
            }
        });

    return { message: `Successfully updated password for user with E-mail ${subordinateEmail}` };
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("reset-password", "reset-password", yargs => {
        yargs.demandOption(['userId', 'subordinateEmail', 'subordinateNewPassword']);
    }, argv => {
        process.env.userInfoTable = "portal-backend-dev-userInfo";
        resetSubordinatePassword({ userId: argv.userId, subordinateEmail: argv.subordinateEmail, subordinateNewPassword: argv.subordinateNewPassword })
            .then(res => console.log(res), err => console.log(err));
    }).argv;
