const axios = require('axios');
const _ = require('lodash');
const bodybuilder = require('bodybuilder');
const program = require("commander");

const KIBANA_URL = "https://cloudcentral-dashboard.tlvengnetapp.com:8080/api/console/proxy";// process.env.KIBANA_PROXY_URL;
const ENDPOINT = "path=audit-prod-*%2F_search&method=POST";

program
    .description("Run the script with client ID and refresh token to monitor the amount of errors in elasticsearch.")
    .option("-c, --client-id [client-id]", "Auth0 client-id")
    .option("-t, --refresh-token [refresh-token]", "Auth0 refresh-token")
    .parse(process.argv);

const assessmentFinishQuery = (gte = "now-7d/d", lt) =>
    bodybuilder()
        .query('match_phrase', 'auditContext.pattern', '/portal/assessment/assessment-finished')
        .query('match_phrase', 'type', 'AUDIT-CLOSE')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_email', '*@netapp.com')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_email', '*@demo.netapp.com')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_email', '*@netapp-test.com')
        .query('range', 'date', { gte, lt })
        .build();

const assessmentFailureQuery = (gte = "now-7d/d", lt) =>
    bodybuilder()
        .query('match_phrase', 'auditContext.pattern', '/portal/assessment/assessment-failed')
        .query('match_phrase', 'type', 'AUDIT-CLOSE')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_email', '*@netapp.com')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_email', '*@demo.netapp.com')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_email', '*@netapp-test.com')
        .query('range', 'date', { gte, lt })
        .build();

const analyticsQuery = (gte = "now-7d/d", lt) =>
    bodybuilder()
        .query('match_phrase', 'auditContext.pattern', '/analytics/event')
        .query('match_phrase', 'type', 'AUDIT-CLOSE')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_userEmail', '*@netapp.com')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_userEmail', '*@demo.netapp.com')
        .notQuery('match_phrase', 'auditContext.requestData.body.s_userEmail', '*@netapp-test.com')
        .query('range', 'date', { gte, lt })
        .build();

const generateSuccessTable = data => {
    const results = data.hits.hits;

    return _.sortBy(_.map(results, result => {
        const datum = result._source.auditContext.requestData.body;
        if (datum.s_provider === 'aws') {
            return {
                Time: new Date(datum.n_reportTime).toISOString(),
                "Cloud Provider": datum.s_provider,
                "User Email": datum.s_email,
                "Total Capacity": datum.a_o_allRegions[0].result.f_totalCapacity,
                "Total Volumes": datum.a_o_allRegions[0].result.totalVolumes,
                "Total Snapshots": datum.a_o_allRegions[0].result.totalSnapshots,
                "Total Saving Cost": datum.a_o_allRegions[0].result.efficiency.f_totalCost
            };
        }
        return {
            Time: new Date(datum.n_reportTime).toISOString(),
            "Cloud Provider": datum.s_provider,
            "User Email": datum.s_email,
            "Total Capacity": datum.o_result.s_f_totalCapacity,
            "Total Volumes": datum.o_result.n_disksCount,
            "Total Snapshots": datum.o_result.n_snapshotsCount,
            "Total Saving Cost": datum.o_result.o_efficiency.s_f_totalCost
        };
    }), 'Time');
};

const percentageDisplay = (part, whole) => `${((part / whole) * 100).toFixed(1)}%`;

const processAnalytics = data => {
    const results = data.hits.hits;
    const actions = _(results)
        .groupBy('_source.auditContext.requestData.body.s_action')
        .mapValues(actionsGroup => _.groupBy(actionsGroup, '_source.auditContext.requestData.body.s_label'))
        .value();

    /* eslint-disable no-multi-spaces, object-curly-newline */
    const flow = [
        { name: 'mainPageViews',            path: 'StartPage["/"]',                            parentPath: null,                                        finalStep: false },
        { name: 'awsPageViews',             path: 'StartPage.aws',                             parentPath: 'StartPage["/"]',                            finalStep: false },
        { name: 'awsAssessmentStarted',     path: 'FlowAction.awsStarted',                     parentPath: 'StartPage.aws',                             finalStep: false },
        { name: 'awsAssessmentFailed',      path: 'FlowAction.awsErrorApi',                    parentPath: 'FlowAction.awsStarted',                     finalStep: true },
        { name: 'awsAssessmentFinished',    path: 'FlowAction.awsSuccessApi',                  parentPath: 'FlowAction.awsStarted',                     finalStep: false },
        { name: 'awsDownloadLists',         path: 'FlowAction.awsDownloadLists',               parentPath: 'FlowAction.awsSuccessApi',                  finalStep: true },
        { name: 'azurePageViews',           path: 'StartPage.azure',                           parentPath: 'StartPage["/"]',                            finalStep: false },
        { name: 'azureLoggedIn',            path: 'FlowAction.azureLogin',                     parentPath: 'StartPage.azure',                           finalStep: false },
        { name: 'azureFetchedSubscription', path: 'FlowAction.fetchAzureSubscriptionsSuccess', parentPath: 'FlowAction.azureLogin',                     finalStep: false },
        { name: 'azureAssessmentStarted',   path: 'FlowAction.azureStarted',                   parentPath: 'FlowAction.fetchAzureSubscriptionsSuccess', finalStep: false },
        { name: 'azureAssessmentSuccess',   path: 'FlowAction.azureSuccess',                   parentPath: 'FlowAction.azureStarted',                   finalStep: false },
        { name: 'azureDownloadLists',       path: 'FlowAction.azureDownloadLists',             parentPath: 'FlowAction.azureSuccess',                   finalStep: true }
    ];
    /* eslint-enable no-multi-spaces, object-curly-newline */

    const actionsUniqueByEmail = _.mapValues(actions, actionsGroup => _.mapValues(actionsGroup, actionsArray => _.uniqBy(actionsArray, '_source.auditContext.requestData.body.s_userEmail')));
    const analyticsResponse = _.map(flow, stage => {
        stage.count = (_.get(actions, stage.path) || []).length;
        stage.countUnique = (_.get(actionsUniqueByEmail, stage.path) || []).length;
        stage.dropOutsList = _.map(_.get(actionsUniqueByEmail, stage.path), '_source.auditContext.requestData.body.s_userEmail'); // Will be updated shortly

        return stage;
    });

    _.forEach(analyticsResponse, stage => {
        const parentStep = _.find(analyticsResponse, { path: stage.parentPath });
        if (parentStep) {
            stage.countAsPrecentage = percentageDisplay(stage.count, parentStep.count);
            parentStep.dropOutsList = _.difference(parentStep.dropOutsList, stage.dropOutsList);
            if (stage.finalStep) {
                stage.dropOutsList = null;
            }
        }
    });

    return _.keyBy(analyticsResponse, 'name');
};

const processFailedAssessments = data => {
    const results = data.hits.hits;

    return _(results)
        .map(result => {
            const datum = result._source.auditContext.requestData.body;
            let message = datum.s_message;
            if (message.includes('not a valid key=value pair ')) {
                message = "The provided AWS credentials are invalid.";
            }

            return {
                Time: new Date(result._source.timestamp).toISOString(),
                "User Email": datum.s_email,
                "Access Key": datum.s_accessKey,
                Message: message
            };
        })
        .sortBy('Time')
        .value();
};

const callKibanaApi = async (query, querySize, cookie, callback) => {
    query.size = querySize;

    return axios.request({
        url: `${KIBANA_URL}?${ENDPOINT}`,
        method: "post",
        headers: {
            "content-type": "application/json",
            cookie,
            "kbn-version": "6.1.1",
            withCredentials: true
        },
        data: query
    })
        .then(res => res.data)
        .then(data => {
            if (data.hits.total > query.size) {
                console.logRed(`response size is limited by query.size. Some results might be missing. Query size: ${query.size}. Total results: ${data.hits.total}`);
            }
            return data;
        })
        .then(callback)
        .catch(err => {
            console.logRed('ERROR!!!');
            console.log(err);
            console.log(err.data);
            console.log(err.data && err.data.error);
        });
};

/*
This function gets an array of jsons and prints it as nice table. Example:
[
    {
        title1: 'a',
        numberValue: 6,
        title3: 'Some long data'
    },
    {
        title1: 'b',
        numberValue: 4.33,
        title3: 'Another long data'
    }
]
==>
title1 | numberValue | title3
a      | 6           | Some long data
b      | 4.33        | Another long data
*/
const printArrayOfJsonsAsTable = (arrayOfJsons, delimiter = ' | ') => {
    const maxValuesLengths = _.mapValues(arrayOfJsons[0], (value, key) => _.max(_.map(arrayOfJsons, json => _.toString(json[key]).length).concat(_.toString(key).length)));

    console.group();
    console.log(_.map(maxValuesLengths, (value, key) => _.toString(key).padEnd(maxValuesLengths[key])).join(delimiter));
    _.forEach(arrayOfJsons, json => console.log(_.map(json, (value, key) => _.toString(value).padEnd(maxValuesLengths[key])).join(delimiter)));
    console.groupEnd();
};

const printReport = () => {
    const cookie = ""; // TODO: refactor to use refresh token and get params as program params.
    const reportTime = new Date();
    const startTime = 1535625947000; // TODO: set the end time of previous report.

    Promise.all([
        callKibanaApi(analyticsQuery(startTime, reportTime.getTime()), 1000, cookie, processAnalytics),
        callKibanaApi(assessmentFinishQuery(startTime, reportTime.getTime()), 500, cookie, generateSuccessTable),
        callKibanaApi(assessmentFailureQuery(startTime, reportTime.getTime()), 500, cookie, processFailedAssessments)
    ])
        .then(([analyticsResponse, successResponse, failureResponse]) => {
            console.log('Weekly Assessment report:');
            console.log('(Note: Users with @netapp.com, @demo.netapp.com and @netapp-test.com e-mail addresses are filtered out)');
            console.log('');
            console.log(`Assessment main page was seen ${analyticsResponse.mainPageViews.count} times.`);
            console.log(`    -> ${analyticsResponse.awsPageViews.count} clicked AWS. (${analyticsResponse.awsPageViews.countAsPrecentage})`);
            console.log(`        -> ${analyticsResponse.awsAssessmentStarted.count} started Assessment. (${analyticsResponse.awsAssessmentStarted.countAsPrecentage})`);
            console.log(`            -> ${analyticsResponse.awsAssessmentFailed.count} failed. (${analyticsResponse.awsAssessmentFailed.countAsPrecentage})`);
            console.log(`            -> ${analyticsResponse.awsAssessmentFinished.count} finished successfully. (${analyticsResponse.awsAssessmentFinished.countAsPrecentage})`);
            console.log(`                -> ${analyticsResponse.awsDownloadLists.count} downloaded the lists. (${analyticsResponse.awsDownloadLists.countAsPrecentage}).`);
            console.log(``);
            console.log(`    -> ${analyticsResponse.azurePageViews.count} clicked Azure. (${analyticsResponse.azurePageViews.countAsPrecentage})`);
            console.log(`        -> ${analyticsResponse.azureLoggedIn.count} did Azure Login. (${analyticsResponse.azureLoggedIn.countAsPrecentage})`);
            console.log(`            -> ${analyticsResponse.azureFetchedSubscription.count} Finished log in and subscription fetching. (${analyticsResponse.azureFetchedSubscription.countAsPrecentage})`);
            console.log(`                -> ${analyticsResponse.azureAssessmentStarted.count} started Assessment. (${analyticsResponse.azureAssessmentStarted.countAsPrecentage})`);
            console.log(`                    -> ${analyticsResponse.azureAssessmentSuccess.count} finished successfully. (${analyticsResponse.azureAssessmentSuccess.countAsPrecentage})`);
            console.log(`                        -> ${analyticsResponse.azureDownloadLists.count} downloaded the lists. (${analyticsResponse.azureDownloadLists.countAsPrecentage}).`);
            console.log(``);

            console.log(`Successful assessments:`);
            printArrayOfJsonsAsTable(successResponse);
            console.log(``);

            // Checking for each failure if it was followed by success.
            _.forEach(failureResponse, failureObject => {
                const userLastSuccess = _.findLast(successResponse, { 'User Email': failureObject['User Email'] });
                const userHasLaterSuccess = userLastSuccess && failureObject.Time.localeCompare(userLastSuccess.Time) === -1;
                failureObject['Followed By Success'] = userHasLaterSuccess ? 'Yes' : 'No';
            });

            console.log(`Failed assessments:`);
            printArrayOfJsonsAsTable(failureResponse);
            console.log(``);

            _.forEach(analyticsResponse, stage => {
                if (stage.dropOutsList && stage.dropOutsList.length) {
                    console.log(`Unique e-mails of users that dropped at stage ${stage.name} (${stage.dropOutsList.length}):`);
                    console.group();
                    console.log(stage.dropOutsList.join(',\n'));
                    console.groupEnd();
                    console.log('');
                }
            });

            console.log(`This report was generated by a script.`);
            console.log(`Report timeframe: ${new Date(startTime).toISOString()} (${startTime}) to ${reportTime.toISOString()} (${reportTime.getTime()})`);
        })
        .catch(console.logRed);
};

const createLogColor = color => (...args) => console.log(color, args);
console.logRed = createLogColor('\x1b[31m%s\x1b[0m'); // See more colors here: https://stackoverflow.com/questions/9781218/how-to-change-node-jss-console-font-color
printReport();
