const _ = require("lodash");
const elasticsearch = require('elasticsearch');
const Promise = require('bluebird');
const AWS = require('aws-sdk');
const moment = require('moment');
const util = require('util');

const { authAxios } = require("./../auth-utils");
const authTypeMapper = require('./auth-type-map');
const { sendInternalEmail } = require('./../utils/internal-email');
const { internalDevTeamMailList } = require("./../consts");

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));

const dynamoDb = new AWS.DynamoDB.DocumentClient();
const elasticClient = new elasticsearch.Client({
    host: process.env.elasticSearchIp,
    apiVersion: '6.0'
});
const retryLimit = 3;
const elasticSearchIndexPrefix = `${process.env.stage}-`;
const getLogsRetryLimit = 10;
const maxLogs = process.env.auth0LegsFetcherMaxLogs;
const maxHoursSinceLastInsert = 12;


const storeAuth0Checkpoint = exports.storeAuth0Checkpoint = (logId, key) => {
    console.log("Storing auth0 checkpoint in dynamo.");
    return dynamoDb.put({
        TableName: process.env.miscTable,
        Item: {
            key,
            value: logId,
            date: new Date().getTime()
        }
    }).promise().then(() => console.log(`Successfully stored auth0 checkpoint in dynamo. Checkpoint is now ${logId}`));
};

const getAuth0Checkpoint = exports.getAuth0Checkpoint = key => {
    console.log("Getting auth0 checkpoint from dynamo");
    return dynamoDb.get({
        Key: {
            key
        },
        TableName: process.env.miscTable
    }).promise().then(({ Item: { value, date } }) => {
        console.log(`Successfully retrieved auth0 logs checkpoint. Latest log id is ${value}`);
        return { value, date };
    });
};

const getLogsRecursive = exports.getLogsRecursive = (acc, logId, count, client, limit = 0, tries = 1) => {
    console.log(`Getting logs from auth0 - recursion (acc length is ${acc.length})`);
    const getLogs = localLogId => client.get('logs', {
        params: {
            take: 100,
            from: localLogId
        }
    });
    const shouldDoAgain = (data, fetchedLogs) => (!(data.length < 100) && (limit === 0 || fetchedLogs.length < limit)); // got 100 logs and still under maxLogs limit
    return getLogs(logId).then(({ data }) => {
        const newAcc = acc.concat(data);
        if (shouldDoAgain(data, newAcc)) {
            const lastIndex = newAcc.length - 1;
            return getLogsRecursive(newAcc, newAcc[lastIndex]._id || newAcc[lastIndex].log_id, count + 1, client, limit, tries);
        }
        console.log(`Successfully retrieved logs from auth0. Retrieved ${newAcc.length} log entries`);
        return newAcc;
    }, err => {
        const errType = _.get(err, "response.status", "unknown");
        if (tries < getLogsRetryLimit) {
            console.log(`Got ${errType} error from Auth0. waiting ${3 * tries} seconds and retrying.... attempt ${tries}/${getLogsRetryLimit}`);
            return Promise.delay(3000 * tries).then(() => getLogsRecursive(acc, logId, count, client, limit, tries + 1));
        }
        console.log(`Reached get logs retry limit...aborting`);
        throw err;
    });
};

const formatLogsForElasticSearch = exports.formatLogsForElasticSearch = logArray => {
    console.log("Formatting logs for insertion into elastic search");
    return new Promise(resolve => {
        const lastEntryId = logArray[logArray.length - 1]._id || logArray[logArray.length - 1].log_id;
        const formattedLogArray = logArray.map(logEntry => {
            // compliance with elastic stored words
            logEntry.id = logEntry._id;
            delete logEntry._id;
            logEntry.auth_type = logEntry.type;
            logEntry.auth_type_text = _.get(authTypeMapper, logEntry.type, "");
            delete logEntry.type;

            // details.error must be an object
            if (_.isString(_.get(logEntry, 'details.error', null))) {
                logEntry.details.error = { message: logEntry.details.error };
            }

            // details.request.body.template must be a string
            if (_.isPlainObject(_.get(logEntry, 'details.request.body.template', null))) {
                logEntry.details.request.body.o_template = logEntry.details.request.body.template;
                logEntry.details.request.body.template = JSON.stringify(logEntry.details.request.body.template);
            }

            // Making sure error payload is object, otherwise Elasticsearch has problems with it.
            const errorPayload = _.get(logEntry, 'details.error.payload');
            if (errorPayload && !_.isPlainObject(errorPayload)) {
                try {
                    const parsedPayload = JSON.parse(errorPayload);
                    logEntry.details.error.payload = _.isPlainObject(parsedPayload) ? parsedPayload : { stringifiedPayload: parsedPayload.toString() };
                } catch (e) {
                    logEntry.details.error.payload = {
                        stringifiedPayload: errorPayload.toString()
                    };
                }
            }

            // Making sure all metadata 'cc:*' permissions are boolean.
            const requestMetadataObject = _.get(logEntry, 'details.request.body.app_metadata', null);
            if (requestMetadataObject) {
                _.forEach(requestMetadataObject, (value, key) => {
                    if (_.startsWith(key, 'cc:') && !_.isBoolean(value)) {
                        delete requestMetadataObject[key];
                        requestMetadataObject[`_illegalKey_${key}`] = JSON.stringify(value);
                        const responseMetadataObject = _.get(logEntry, 'details.response.body.app_metadata', null);
                        if (responseMetadataObject) {
                            delete responseMetadataObject[key];
                            responseMetadataObject[`_illegalKey_${key}`] = JSON.stringify(value);
                        }
                    }
                });
            }
            return logEntry;
        });

        const groups = _.groupBy(formattedLogArray, logEntry => { // key is the _index in elastic
            const date = new Date(logEntry.date);
            return `auth-${elasticSearchIndexPrefix}${date.getUTCFullYear()}.${date.getUTCMonth() + 1}.${date.getUTCDate()}`;
        });

        console.log("Done formatting logs for insertion into elastic search");
        resolve({ lastEntryId, groups });
    });
};

const insertRetrier = ({ index, body, retryCount }) => {
    const promise = elasticClient.bulk({ index, type: 'auth0', body });
    return promise.then(response => {
        if (response.errors) {
            console.log("Failed to push specific auth0 logs to elastic search");
            if (response.items.find(el => _.get(el, 'create.error', null) && _.get(el, 'create.error.type', null) && el.create.error.type !== "version_conflict_engine_exception")) {
                console.log("Errors:");
                console.log(util.inspect(response.items.filter(resp => _.get(resp, 'create.error', null) && resp.create.error.type !== "version_conflict_engine_exception"), { showHidden: true, depth: null }));
                if (retryCount < retryLimit) {
                    console.log(`Insertion to elastic search returned an error...retrying... retry count: ${retryCount + 1}`);
                    return insertRetrier({ index, body, retryCount: retryCount + 1 });
                }
                console.log(`Reached insertion retry limit of ${retryLimit}. Insertion aborted. NOT updating dynamo with auth0 checkpoint`);
                const failures = response.items.filter(el => _.get(el, 'create.error', null) && _.get(el, 'create.error.type', null) && el.create.error.type !== "version_conflict_engine_exception");
                failures.forEach(failure => console.log(failure));
                const message = `Failed to push Auth0 logs in ${process.env.stage} env. See cloud watch logs for more details`;
                return sendInternalEmail("Failed to push Auth0 logs to Elastic", message, internalDevTeamMailList).then(() => Promise.reject(new Error(`Reached insertion retry limit of ${retryLimit}. Insertion aborted. NOT updating dynamo with auth0 checkpoint`)));
            }
            console.log("Error of type 'version_conflict_engine_exception' (log entry already exists), not retrying");
            return response;
        } return response;
    });
};

const insertLogsIntoElasticSearch = ({ lastEntryId, groups }) => {
    console.log("Inserting logs into elastic search");
    return Promise.all(_.map(groups, (val, key) => {
        const requestArray = _.flatten(val.map(x => [{ create: { _id: x.id || x.log_id } }, x])); // add elastic create command before each element
        return insertRetrier({ index: key, body: requestArray, retryCount: 0 });
    })).then(() => {
        console.log("Insertion to elastic completed successfully");
        return lastEntryId;
    });
};

const checkTimeSinceLastInsert = date => {
    const diff = moment().diff(moment(date), "hours");
    console.log(`Hours since last insert: ${diff} (expected val = 0)`);
    const message = `Time since last insert of auth0 logs to elastic (${diff}) in ${process.env.stage} env exceeded limit of ${maxHoursSinceLastInsert} hours`;
    if (diff > maxHoursSinceLastInsert && process.env.stage !== 'dev') {
        sendInternalEmail("Time since last insert to elastic search exceeded threshold", message, internalDevTeamMailList);
    }
};

const handler = exports.handler = (event, context, callback) => {
    console.log("Attempting to push Auth0 logs to Elastic Search");

    return getAuth0Checkpoint('auth0Checkpoint').then(({ value, date }) => {
        console.log("Getting logs from auth0");
        checkTimeSinceLastInsert(date);
        return authAxios().then(client => getLogsRecursive([], value, 0, client, maxLogs)
            .then(logArray => {
                if (logArray.length === 0) {
                    console.log("No new logs from Auth0. Exiting.");
                    return null;
                }
                return formatLogsForElasticSearch(logArray)
                    .then(insertLogsIntoElasticSearch)
                    .then(lastEntryId => storeAuth0Checkpoint(lastEntryId, 'auth0Checkpoint'));
            })
            .then(() => {
                console.log("Success!");
                callback(null, {});
            }));
    }).catch(err => {
        console.log("Error");
        console.log(err);
        return callback(null, {});
    });
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("time", "time", _.noop, () => {
        checkTimeSinceLastInsert(1524644065439);
    })
    .command("test", "test", _.noop, () => {
        handler();
    }).argv;
