const { authAxios } = require("./auth-utils");
const { getUsers } = require("./common");

const getUsersWrapper = exports.getUsersWrapper = ({ fields, q }) => {
    console.log(`Attempting to get all users for query ${q}`);

    return getUsers({ params: { fields, q } });
};

const getGuardianEnrollments = exports.getGuardianEnrollments = async ({ userId }) => {
    console.log(`Attempting to get list of guardian enrollments for user ${userId}`);

    const authClient = await authAxios();
    try {
        const { data: enrollments } = await authClient.get(`users/${userId}/enrollments`);
        console.log(`Successfully got ${enrollments.length} enrollments.`);

        return enrollments;
    } catch (err) {
        console.log(`Failed to get list of guardian enrollments for user ${userId}`, err);

        throw err;
    }
};

const deleteGuardianEnrollment = exports.deleteGuardianEnrollment = async ({ id: enrollmentId }) => {
    console.log(`Attempting to delete guardian enrollment ${enrollmentId}`);

    const authClient = await authAxios();
    try {
        const response = await authClient.delete(`guardian/enrollments/${enrollmentId}`);
        console.log(`Successfully deleted enrollment ${enrollmentId}.`);

        return response;
    } catch (err) {
        console.log(`Failed to delete guardian enrollment ${enrollmentId}`, err);

        throw err;
    }
};

const resetMfa = exports.resetMfa = async ({ userId }) => {
    console.log(`Attempting to reset MFA for user ${userId}`);

    try {
        const enrollments = await getGuardianEnrollments({ userId });

        return Promise.all(enrollments.map(deleteGuardianEnrollment))
            .then(() => console.log(`Successfully deleted ${enrollments.length} enrollments.`));
    } catch (err) {
        console.log(`Failed to reset MFA for user ${userId}`, err);

        throw err;
    }
};

const getUserBlocks = exports.getUserBlocks = async ({ userId }) => {
    console.log(`Attempting to get all blocks for user ${userId}`);

    const authClient = await authAxios();
    try {
        const { data: { blocked_for: blockedFor } } = await authClient.get(`user-blocks/${userId}`);
        console.log(`Successfully retrieved all ${blockedFor.length} blocks.`);

        return blockedFor;
    } catch (err) {
        console.log(`Failed to retrieve blocks for user.`, err);

        throw err;
    }
};

const unblockUser = exports.unblockUser = async ({ userId }) => {
    console.log(`Attempting to unblock user ${userId}`);

    const authClient = await authAxios();
    try {
        await authClient.delete(`user-blocks/${userId}`);
        console.log(`Successfully unblocked user ${userId}.`);
    } catch (err) {
        console.log(`Failed to unblocked user ${userId}.`, err);

        throw err;
    }
};

const generatePasswordChangeTicket = exports.generatePasswordChangeTicket = async ({ userId }) => {
    console.log(`Attempting to generate password change ticket for user ${userId}.`);
    const stagePrefix = process.env.stage === 'production' ? '' : `${process.env.stage}.`;
    const resultUrl = `https://${stagePrefix}services.cloud.netapp.com/callback`;

    const authClient = await authAxios();
    try {
        const { data } = await authClient.post(`/tickets/password-change`, {
            result_url: resultUrl,
            user_id: decodeURIComponent(userId)
        });

        console.log(`Successfully generated password change ticket for user ${userId}.`);

        return { resetPasswordUrl: data.ticket };
    } catch (err) {
        console.log(`Failed to generated password change ticket for user ${userId}.`, err);

        throw err;
    }
};

const getUserLogs = exports.getUserLogs = async ({ userId, page, perPage = 50 }) => {
    console.log(`Attempting to get logs for user ${userId}. (page: ${page}, perPage: ${perPage})`);

    const authClient = await authAxios();
    try {
        const { data } = await authClient.get(`/users/${userId}/logs`, {
            params: {
                include_totals: true,
                page,
                per_page: perPage
            }
        });
        console.log(`Successfully retrieved ${data.length} log entries for user ${userId}. (total: ${data.total}, start: ${data.start}, limit: ${data.limit})`);

        return data;
    } catch (err) {
        console.log(`Failed to retrieve logs for user ${userId}.`, err);

        throw err;
    }
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("get-users", "get-users", yargs => {
        yargs.demandOption(['q']);
    }, argv => {
        getUsersWrapper({ fields: argv.fields, q: argv.q })
            .then(console.log);
    })
    .command("get-user-blocks", "get-user-blocks", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        getUserBlocks({ userId: argv.userId })
            .then(console.log);
    })
    .command("get-guardian-enrollments", "get-guardian-enrollments", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        getGuardianEnrollments({ userId: argv.userId })
            .then(console.log);
    })
    .command("reset-mfa", "reset-mfa", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        resetMfa({ userId: argv.userId })
            .then(console.log);
    })
    .command("unblock-user", "unblock-user", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        unblockUser({ userId: argv.userId })
            .then(({ data }) => console.log(data))
            .catch(console.log);
    })
    .command("generate-password-change-ticket", "generate-password-change-ticket", yargs => {
        yargs.demandOption(['userId']);
    }, argv => {
        generatePasswordChangeTicket({ userId: argv.userId })
            .then(({ data }) => console.log(data))
            .catch(console.log);
    })
    .command("get-user-logs", "get-user-logs", yargs => {
        yargs.demandOption(['userId']);
        yargs.demandOption(['page']);
    }, argv => {
        getUserLogs({ userId: argv.userId, page: argv.page, perPage: argv.perPage })
            .then(console.log)
            .catch(console.log);
    }).argv;
