const AWS = require('aws-sdk');
const _ = require('lodash');

const { getUsers } = require("./../common");
const { ClientError } = require("./../errors");
const { authAxios } = require("./../auth-utils");

AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));

const getAllConnections = exports.getAllConnections = ({ fields }) => {
    console.log(`Attempting to get all connections.`);
    return authAxios()
        .then(client => client.get('connections', { params: { fields } })
            .then(({ data }) => {
                console.log(`Successfully got list of ${data.length} connections`);
                return data;
            }, err => {
                console.log(`Error. Getting list of connections failed. `, err);
                throw err;
            }));
};

const getConnectionIdByName = async ({ connectionName }) => {
    console.log(`Getting connection id by name: ${connectionName}`);

    const connections = await getAllConnections({ fields: "name" });
    const foundConnection = _.find(connections, { name: connectionName });

    if (foundConnection) {
        return foundConnection.id;
    }
    throw new ClientError(`Could not find connection ${connectionName}.`);
};

const getAllRulesConfigKeys = () => {
    console.log(`Attempting to get all rules config keys.`);
    return authAxios()
        .then(client => client.get('rules-configs')
            .then(({ data }) => {
                console.log(`Successfully got list of rules config keys`);
                return data;
            }, err => {
                console.log(`Error. Getting list of rules config keys failed. `, err);
                throw err;
            }));
};

exports.getAllConnectionsWithRulesInfo = ({ fields }) => Promise.all([
    getAllConnections({ fields }),
    getAllRulesConfigKeys()
])
    .then(([connectionsData, rulesConfigData]) =>
        connectionsData.map(connection => {
            connection.hasRuleConfig = Boolean(_.find(rulesConfigData, { key: connection.name }));
            return connection;
        }));

const addConfigValueForFederationCompanyName = ({ connectionName, companyName }) => {
    console.log(`Adding config value for federation company name for connection with name: ${connectionName}`);

    return authAxios()
        .then(client => client.put(`rules-configs/${connectionName}`, { value: companyName })
            .then(res => {
                console.log(`Successfully set the rules config`);
                return res;
            }, err => {
                console.error(`Error. Setting rules config failed`, err);
                throw err;
            }));
};

const getAllNonOccmClientsFromDynamoDb = () => {
    console.log(`Attempting to get non-OCCM clients from dynamoDB table.`);

    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const connectionParams = {
        TableName: process.env.staticClientTable,
        FilterExpression: 'attribute_exists(displayName)'
    };

    return dynamoDb.scan(connectionParams).promise().then(({ Items }) => {
        console.log(`Got all ${Items.length} clients`);
        return Items;
    }, err => {
        console.log("Failed to retrieve all non-OCCM clients from dynamoDB", err);
        throw err;
    });
};

const getGeneralClientsIdsToEnable = () =>
    getAllNonOccmClientsFromDynamoDb()
        .then(notOccmClients => {
            const excludedClients = (process.env.netAppInternalClients || '').split(',');

            return _(notOccmClients)
                .reject(client => excludedClients.includes(client.clientName))
                .map('clientId')
                .value();
        });

const getUsersIdsByEmailDomains = async ({ emailDomains }) => {
    console.log(`Attempting to get all users with email domains ${emailDomains}`);

    // query: 'a.com,b.com' => 'email:*@a.com OR email:*@b.com'
    const query = emailDomains.split(',').map(emailDomain => `email:*@${emailDomain}`).join(' OR ');

    return getUsers({ params: { fields: 'user_id', q: query } })
        .then(users => _.map(users, 'user_id'));
};

const getUsersOccmsIdsByUsersIds = ({ usersIds }) => {
    console.log(`Attempting to get list of OCCMs that are attached to list of ${usersIds.length} users from dynamoDB table.`);

    if (usersIds.length === 0) {
        return Promise.resolve([]);
    }

    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const params = {
        RequestItems: {
            [process.env.userInfoTable]: {
                Keys: _.map(usersIds, userId => ({ userId })),
                ProjectionExpression: "associatedOccms"
            }
        }
    };

    return dynamoDb.batchGet(params).promise().then(({ Responses }) => {
        const occmsIds = _(Responses[process.env.userInfoTable])
            .map('associatedOccms')
            .flatMap('values')
            .compact()
            .value();
        console.log(`Got OCCMs data for ${Responses[process.env.userInfoTable].length} users with ${occmsIds.length} OCCMs`);

        return occmsIds;
    }, err => {
        console.log("Failed to get list of OCCMs that are attached to list of users from dynamoDB table.", err);
        throw err;
    });
};

const getAllOccmClientIdsAssociatedWithEmailDomains = ({ emailDomains }) =>
    getUsersIdsByEmailDomains({ emailDomains })
        .then(usersIds => getUsersOccmsIdsByUsersIds({ usersIds }));

const enableRelevantClientsOnConnection = exports.enableRelevantClientsOnConnection = ({ connectionId, emailDomains }) => {
    console.log(`Enabling all relevant clients for connection with id: ${connectionId}`);
    return authAxios()
        .then(authClient =>
            Promise.all([
                getGeneralClientsIdsToEnable(),
                getAllOccmClientIdsAssociatedWithEmailDomains({ emailDomains })
            ])
                .then(([generalClientsIds, occmClientIds]) =>
                    authClient.patch(`connections/${connectionId}`, { enabled_clients: generalClientsIds.concat(occmClientIds) })
                        .then(() => {
                            const occmClientsMessage = occmClientIds.length ? `${occmClientIds.length} OCCM clients (OCCM ids: ${occmClientIds.join(', ')}) associated with users with email domains ${emailDomains}` : `0 OCCM clients`;
                            const messagePrefix = `Successfully enabled all ${generalClientsIds.length} general clients and ${occmClientsMessage}`;
                            console.log(`${messagePrefix} for connection with id ${connectionId}`);

                            return { generalClientsCount: generalClientsIds.length, occmClientIds };
                        }, err => {
                            console.log(`Failed to enable all general clients for connection with id ${connectionId}`, err);
                            throw err;
                        })));
};

const enableConnection = exports.enableConnection = async ({ connectionName, companyName, emailDomains }) => {
    console.log(`Enabling connection ${connectionName} for company ${companyName}`);

    const connectionId = await getConnectionIdByName({ connectionName });
    const enableClientsResponse = await enableRelevantClientsOnConnection({ connectionId, emailDomains });
    await addConfigValueForFederationCompanyName({ connectionName, companyName });

    return {
        connectionName,
        companyName,
        emailDomains,
        connectionId,
        generalClientsCount: enableClientsResponse.generalClientsCount,
        occmClientIds: enableClientsResponse.occmClientIds
    };
};

const disableAllClientsOnConnection = exports.enableAllGeneralClientsOnConnection = ({ connectionId }) => {
    console.log(`Disabling all clients for connection with id: ${connectionId}`);
    return authAxios()
        .then(authClient => authClient.patch(`connections/${connectionId}`, { enabled_clients: [] })
            .then(() => {
                console.log(`Successfully disabled all clients for connection with id ${connectionId}`);
            }, err => {
                console.log(`Failed to disable all clients for connection with id ${connectionId}`, err);
                throw err;
            }));
};

const deleteConfigValueForFederationCompanyName = ({ connectionName }) => {
    console.log(`Deleting config value for federation company name for connection with name: ${connectionName}`);

    return authAxios()
        .then(client => client.delete(`rules-configs/${connectionName}`)
            .then(res => {
                console.log(`Successfully deleted the rules config`);
                return res;
            }, err => {
                console.error(`Error. Deleting rules config failed`, err);
                throw err;
            }));
};

const disableConnection = exports.disableConnection = async ({ connectionName }) => {
    console.log(`Disabling connection ${connectionName}`);

    const connectionId = await getConnectionIdByName({ connectionName });
    await disableAllClientsOnConnection({ connectionId });
    await deleteConfigValueForFederationCompanyName({ connectionName });

    return { message: `Disabled connection "${connectionName}" successfully.` };
};

const yarger = require("yargs") // eslint-disable-line no-unused-vars, import/order
    .command("enable-connection", "enable-connection", yargs => {
        yargs.demandOption(["connectionName", "companyName", "emailDomains"]);
    }, argv => {
        if (!argv.connectionName) { throw new Error("Missing required parameter connectionName"); }
        if (!argv.companyName) { throw new Error("Missing required parameter companyName"); }
        if (!argv.emailDomains) { throw new Error("Missing required parameter emailDomains"); }
        if (!process.env.managementDomain) { throw new Error("No management settings in variables"); }
        enableConnection({ connectionName: argv.connectionName, companyName: argv.companyName, emailDomains: argv.emailDomains })
            .catch(() => process.exit(1));
    })
    .command("disable-connection", "disable-connection", yargs => {
        yargs.demandOption(["connectionId"]);
    }, argv => {
        if (!argv.connectionName) { throw new Error("Missing required parameter connectionName"); }
        if (!process.env.managementDomain) { throw new Error("No management settings in variables"); }
        disableConnection({ connectionName: argv.connectionName })
            .catch(() => process.exit(1));
    })
    .command("get-all-non-occm-clients-from-dynamo-db", "get-all-non-occm-clients-from-dynamo-db", _.noop, () => {
        if (!process.env.staticClientTable) { throw new Error("No static client table settings in variables"); }
        if (!process.env.managementDomain) { throw new Error("No management settings in variables"); }
        getAllNonOccmClientsFromDynamoDb()
            .then(console.log)
            .catch(() => process.exit(1));
    })
    .command("get-users-ids-by-email-domains", "get-users-ids-by-email-domains", yargs => {
        yargs.demandOption(["emailDomains"]);
    }, argv => {
        if (!argv.emailDomains) { throw new Error("Missing required parameter emailDomains"); }
        if (!process.env.managementDomain) { throw new Error("No management settings in variables"); }
        getUsersIdsByEmailDomains({ emailDomains: argv.emailDomains })
            .then(console.log)
            .catch(() => process.exit(1));
    })
    .command("get-all-connections", "get-all-connections", yargs => {
        yargs.demandOption(["fields"]);
    }, argv => {
        if (!process.env.managementDomain) { throw new Error("No management settings in variables"); }
        getAllConnections({ fields: argv.fields })
            .then(console.log)
            .catch(() => process.exit(1));
    }).argv;
