const chai = require("chai");
const expect = require('chai').expect;
const chaiAsPromised = require("chai-as-promised");
chai.use(chaiAsPromised);
const _ = require("lodash");
const testLinkAndCollect = require('./visiting').testLinkAndCollect;
const login = require('./browser').login;
const puppeteerBrowser = require('./browser').browser;
const merge = require('./merge').merge;

describe('testLinkAndCollect function test', function() {

    it('Should verify and NOT collect hrefs from non whitelisted URL', async function () {
        const browser = await puppeteerBrowser();
        const res = await testLinkAndCollect(browser, {url: "https://www.google.com/", parent: "https://cloud.netapp.com"}, true);
        await browser.close();
        // noinspection BadExpressionStatementJS
        expect(res.hrefs).to.be.empty;
        console.log(JSON.stringify(res));
    });

    it('Should verify and collect hrefs from whitelisted URL: cloud.netapp.com', async function () {
        const browser = await puppeteerBrowser();
        const res = await testLinkAndCollect(browser, {url: "https://cloud.netapp.com/cloud-sync-service", parent: "https://cloud.netapp.com"}, true);
        await browser.close();
        // noinspection BadExpressionStatementJS
        expect(res.hrefs).to.not.be.empty;
    });

    it('Should NOT verify and collect hrefs from whitelisted URL: cloud.netapp.com if collect=false', async function () {
        const browser = await puppeteerBrowser();
        const res = await testLinkAndCollect(browser, {url: "https://cloud.netapp.com/cloud-sync-service", parent: "https://cloud.netapp.com"}, false);
        await browser.close();
        // noinspection BadExpressionStatementJS
        expect(res.hrefs).to.be.empty;
    });

    it('Should verify and collect hrefs from whitelisted URL: services.cloud.netapp.com', async function () {
        const browser = await puppeteerBrowser();
        await login(browser, "https://services.cloud.netapp.com/?redirectToLogin=true");
        const res = await testLinkAndCollect(browser, {url: "https://services.cloud.netapp.com/", parent: "https://cloud.netapp.com"}, true);
        await browser.close();
        // noinspection BadExpressionStatementJS
        expect(res.hrefs).to.not.be.empty;
    });

    it('Should not collect hrefs from broken URL', async function () {
        const browser = await puppeteerBrowser();
        const res = await testLinkAndCollect(browser, {url: "https://cloud.netapp.com/stam", parent: "https://cloud.netapp.com"}, true);
        await browser.close();
        // noinspection BadExpressionStatementJS
        expect(res.hrefs).to.be.empty;
        expect(res.isBroken).to.equal(true);
    });

    it('Should not collect hrefs from staging URL', async function () {
        const browser = await puppeteerBrowser();
        const res = await testLinkAndCollect(browser, {url: "https://cloud.netapp.com/staging", parent: "https://cloud.netapp.com"}, true);
        await browser.close();
        // noinspection BadExpressionStatementJS
        expect(res.hrefs).to.be.empty;
        expect(res.isStaging).to.equal(true);
    });

});

describe('merge function test', function() {
    it('Should add any broken URLs to output', async function () {
        let output = {
            notVisited: [{url: 'https://rooturl.com', parentUrl: undefined}],
            visited: [],
            broken: {},
            staging: {},
            puppeteerError: []
        };

        const batchResults = [
            {
                rootLink: {url: 'https://correct.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://broken1.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: true,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://broken2.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: true,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://staging.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: true,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://error.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: true
            }
        ];
        merge(output, batchResults);
        const expectedRes = {};
        expectedRes[batchResults[1].rootLink.url] = [batchResults[1].rootLink.parentUrl];
        expectedRes[batchResults[2].rootLink.url] = [batchResults[2].rootLink.parentUrl];
        expect(output.broken).to.eql(expectedRes);
    });

    it('Should add any staging URLs to output', async function () {
        let output = {
            notVisited: [{url: 'https://rooturl.com', parentUrl: undefined}],
            visited: [],
            broken: {},
            staging: {},
            puppeteerError: []
        };

        const batchResults = [
            {
                rootLink: {url: 'https://correct.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://staging1.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: true,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://staging2.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: true,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://broken.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: true,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://error.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: true
            }
        ];
        merge(output, batchResults);
        const expectedRes = {};
        expectedRes[batchResults[1].rootLink.url] = [batchResults[1].rootLink.parentUrl];
        expectedRes[batchResults[2].rootLink.url] = [batchResults[2].rootLink.parentUrl];
        expect(output.staging).to.eql(expectedRes);
    });

    it('Should add any error URLs to output', async function () {
        let output = {
            notVisited: [{url: 'https://rooturl.com', parentUrl: undefined}],
            visited: [],
            broken: {},
            staging: {},
            puppeteerError: []
        };

        const batchResults = [
            {
                rootLink: {url: 'https://correct.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://error1.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: true
            },
            {
                rootLink: {url: 'https://error2.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: true
            },
            {
                rootLink: {url: 'https://broken.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: true,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://staging.com', parentUrl: 'https://rooturl.com'},
                hrefs: [],
                isBroken: false,
                isStaging: true,
                isPuppeteerError: false
            }
        ];
        merge(output, batchResults);
        expect(output.puppeteerError).to.eql([batchResults[1].rootLink, batchResults[2].rootLink]);
    });

    it('Should gather batchResult hrefs that were not yet visited and add them uniquely to not visited', async function () {
        let output = {
            notVisited: [],
            visited: [],
            broken: {},
            staging: {},
            puppeteerError: []
        };

        const batchResults = [
            {
                rootLink: {url: 'https://visited1.com', parentUrl: 'https://rooturl.com'},
                hrefs: [
                    {url: 'https://duplicated.com', parentUrl: 'https://visited1.com'},
                    {url: 'https://duplicated.com', parentUrl: 'https://visited1.com'},
                    {url: 'https://foundAlsoInOtherPage.com', parentUrl: 'https://visited1.com'},
                ],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://visited2.com', parentUrl: 'https://rooturl.com'},
                hrefs: [
                    {url: 'https://foundAlsoInOtherPage.com', parentUrl: 'https://visited2.com'},
                    {url: 'https://foundOnce1.com', parentUrl: 'https://visited2.com'},
                    {url: 'https://foundOnce2.com', parentUrl: 'https://visited2.com'},
                ],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            }
        ];
        merge(output, batchResults);
        expect(_.map(output.notVisited, 'url')).to.eql(['https://duplicated.com','https://foundAlsoInOtherPage.com','https://foundOnce1.com', 'https://foundOnce2.com']);
    });

    it('Should gather batchResult hrefs that were not yet visited and filter the visited ones', async function () {
        let output = {
            notVisited: [],
            visited: [
                {url: 'https://visited1.com', parentUrl: 'https://parent1.com'},
                {url: 'https://visited2.com', parentUrl: 'https://parent2.com'},
                {url: 'https://visited3.com', parentUrl: 'https://parent3.com'}
                ],
            broken: {},
            staging: {},
            puppeteerError: []
        };

        const batchResults = [
            {
                rootLink: {url: 'https://visited1.com', parentUrl: 'https://parent1.com'},
                hrefs: [
                    {url: 'https://visited1.com', parentUrl: 'https://visited1.com'},
                    {url: 'https://notVisited1.com', parentUrl: 'https://visited1.com'},
                    {url: 'https://notVisited2.com', parentUrl: 'https://visited1.com'},
                ],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://visited2.com', parentUrl: 'https://parent2.com'},
                hrefs: [
                    {url: 'https://visited2.com', parentUrl: 'https://visited2.com'},
                    {url: 'https://visited3.com', parentUrl: 'https://visited2.com'},
                    {url: 'https://notVisited3.com', parentUrl: 'https://visited2.com'},
                ],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            }
        ];
        merge(output, batchResults);
        expect(_.map(output.notVisited, 'url')).to.eql(['https://notVisited1.com','https://notVisited2.com','https://notVisited3.com']);
    });

    it('Should add parent url of hrefs that were identified as broken in previous visit steps', async function () {
        let output = {
            notVisited: [],
            visited: [],
            broken: {'https://broken1.com': ['https://parent1.com']},
            staging: {},
            puppeteerError: []
        };

        const batchResults = [
            {
                rootLink: {url: 'https://visited1.com', parentUrl: 'https://parent1.com'},
                hrefs: [
                    {url: 'https://broken1.com', parentUrl: 'https://visited1.com'}
                ],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://visited2.com', parentUrl: 'https://parent2.com'},
                hrefs: [
                    {url: 'https://broken1.com', parentUrl: 'https://visited2.com'}
                ],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            }
        ];
        merge(output, batchResults);
        expect(output.broken).to.eql({'https://broken1.com': ['https://parent1.com', batchResults[0].rootLink.url, batchResults[1].rootLink.url]});
    });

    it('Should add parent url of hrefs that were identified as staging in previous visit steps', async function () {
        let output = {
            notVisited: [],
            visited: [],
            broken: {},
            staging: {'https://staging1.com': ['https://parent1.com'], 'https://staging2.com': ['https://parent2.com']},
            puppeteerError: []
        };

        const batchResults = [
            {
                rootLink: {url: 'https://visited1.com', parentUrl: 'https://parent1.com'},
                hrefs: [
                    {url: 'https://staging1.com', parentUrl: 'https://visited1.com'}
                ],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            },
            {
                rootLink: {url: 'https://visited2.com', parentUrl: 'https://parent2.com'},
                hrefs: [
                    {url: 'https://staging2.com', parentUrl: 'https://visited2.com'}
                ],
                isBroken: false,
                isStaging: false,
                isPuppeteerError: false
            }
        ];
        merge(output, batchResults);
        expect(output.staging).to.eql({'https://staging1.com': ['https://parent1.com', batchResults[0].rootLink.url], 'https://staging2.com': ['https://parent2.com', batchResults[1].rootLink.url]});
    });
});

