const _ = require('lodash');
const AwsConfigClient = require('../utils/aws/AwsConfigClient');
const Constants = require('../utils/Constants');
const EC2Client = require('../utils/aws/EC2Client');
const Promise = require('bluebird');
const logger = require('../logger/Logger');

const tageName = 'assessment';

const generateInstanceRequest = () => {
    return params = {
        MaxCount: 1, /* required */
        MinCount: 1, /* required */
        BlockDeviceMappings: [
            {
                DeviceName: '/dev/sda1',
                Ebs: {
                    DeleteOnTermination: true,
                    VolumeSize: 10,
                    VolumeType: 'gp2'
                },
            },
            {
                DeviceName: '/dev/sdb',
                Ebs: {
                    DeleteOnTermination: true,
                    Encrypted: false,
                    VolumeSize: 10,
                    VolumeType: 'gp2'
                },
            },
        ],
        DisableApiTermination: false,
        EbsOptimized: false,
        ImageId: Constants.sydneyRedHatAmi,
        InstanceType: "t1.micro",
        Monitoring: {
            Enabled: false /* required */
        },
        TagSpecifications: [
            {
                ResourceType: 'instance',
                Tags: [
                    {
                        Key: 'Name',
                        Value: tageName
                    },
                ]
            },
        ],
    }
};

const generateSnapshotRequest = (volumeId) => {
  return  {
        Description: "This is my root volume snapshot.",
        VolumeId: volumeId,
        TagSpecifications: [
            {
                ResourceType: 'snapshot',
                Tags: [
                    {
                        Key: 'Name',
                        Value: tageName
                    }
                ]
            }
        ]
    };
};

const generateVolumeRequest = () => {
    return  {
        AvailabilityZone: Constants.sydneyAvailabilityZone,
        VolumeType: 'gp2',
        Size: 10,
        TagSpecifications: [
            {
                ResourceType: 'volume',
                Tags: [
                    {
                        Key: 'Name',
                        Value: tageName
                    }
                ]
            }
        ]
    };
};

const createEnv = async () => {
    logger.info("Start creating environment");
    const envConfig = {};
    const awsConfig = new AwsConfigClient();
    awsConfig.setRegion(Constants.sydney);
    const ec2 = new EC2Client(logger);
    const ec2Params = generateInstanceRequest();
    try {
        envConfig.Instnace = await ec2.createRunningInstance(ec2Params);
        logger.info("Finished creating a running instance with 2 attached volumes");
        const volumeRequest1 = ec2.createAvailableVolume(generateVolumeRequest());
        const volumeRequest2 = ec2.createAvailableVolume(generateVolumeRequest());
        const volumeRequest3 = ec2.createAvailableVolume(generateVolumeRequest());
        await Promise.all([volumeRequest1, volumeRequest2, volumeRequest3]);
        logger.info("Finished creating 3 volumes");
        const volumesInfo = await ec2.describeVolumes();
        const attached = volumesInfo.filter(volumeInfo => !_.isEmpty(volumeInfo.Attachments));
        const unattached = volumesInfo.filter(volumeInfo => _.isEmpty(volumeInfo.Attachments));
        const attachedVolumeIdForSnapshot = attached[0].VolumeId;
        const unattachedVolumeIdForSnapshot = unattached[0].VolumeId;
        const volumeIdForUnattachedSnapshot = unattached[1].VolumeId;
        const snapshotRequest1 = ec2.createCompletedSnapshot(generateSnapshotRequest(attachedVolumeIdForSnapshot));
        const snapshotRequest2 = ec2.createCompletedSnapshot(generateSnapshotRequest(unattachedVolumeIdForSnapshot));
        const snapshotRequest3 = ec2.createCompletedSnapshot(generateSnapshotRequest(volumeIdForUnattachedSnapshot));
        await Promise.all([snapshotRequest1, snapshotRequest2, snapshotRequest3]);
        await ec2.deleteVolumeById(volumeIdForUnattachedSnapshot);
        logger.info("Finished creating a snapshot for attached volume, a snapshot for unattached volume and unattached snapshot");
        await Promise.delay(5000);
    } catch (e) {
        logger.error(e);
    }
};

const cleanEnv = async () => {
    const awsConfig = new AwsConfigClient();
    awsConfig.setRegion(Constants.sydney);
    const ec2 = new EC2Client();
    const params = {
        Filters: [
            {
                Name: 'tag-value',
                Values: [
                    tageName,
                ]
            },
        ]
    };
    try {
        const instances = await ec2.describeInstances(params);
        logger.info(`Instances to delete: ${JSON.stringify(instances)}`);
        const instancesToDelete = instances.map(instanceInfo => instanceInfo.Instances[0].InstanceId);
        await ec2.deleteInstancesById(instancesToDelete);

        const volumesToDelete = await ec2.describeVolumes(params);
        logger.info(`Volumes to delete: ${JSON.stringify(volumesToDelete)}`);
        Promise.map(volumesToDelete, async (volumeInfo) => {
            try {
                const volumeId = volumeInfo.VolumeId;
                await ec2.deleteVolumeById(volumeId);
            } catch (error) {
                return console.log(error);
            }
        }, {concurrency: 2});

        const snapshotsToDelete = await ec2.describeSnapshots(params);
        logger.info(`Snapshots to delete: ${JSON.stringify(snapshotsToDelete)}`);
        Promise.map(snapshotsToDelete, async (snapshotInfo) => {
            try {
                const snapshotId = snapshotInfo.SnapshotId;
                await ec2.deleteSnapshotById(snapshotId);
            } catch (error) {
                logger.error(error);
            }
        }, {concurrency: 2});
    }catch (e) {
        logger.error(e);
    }
};

// createEnv().catch(console.log);
cleanEnv().catch(console.log);









