const axios = require('axios');
const Promise = require('bluebird');
const Initializer = require('../utils/specInitializer');
const Constants = require('../utils/Constants');
const AWS = require('aws-sdk');
// eslint-disable-line import/no-extraneous-dependencies
AWS.config.update({ region: 'us-east-1' });
AWS.config.setPromisesDependency(require('bluebird'));
const _ = require("lodash");

const { logger, clients } = Initializer('OCCM cleanup script');

const authAccount = exports.authAccount = {
    domain: process.env.managementDomain,
    godClientId: process.env.managementClientId,
    godClientSecret: process.env.managementClientSecret
};

const authAxios = _.once(() => {
    const instance = axios.create({
        baseURL: `https://${authAccount.domain}/`
    });

    const audience = `https://${authAccount.domain}/api/v2/`;

    instance.interceptors.response.use(response => {
        return response;
    }, err => {
        if (err.statusCode === 429 && err.config && (!err.config.__retryCount || err.config.__retryCount < 3)) {
            err.config.__retryCount = err.config.__retryCount ? err.config.__retryCount + 1 : 1;
            return new Promise((resolve, reject) => {
                console.log("Call failed with 429 - retrying in 3000ms");
                setTimeout(() => {
                    axios(err.config).then(resolve, reject);
                }, 3000);
            });
        }
        throw err;
    });

    return instance.post(`oauth/token`, {
        grant_type: "client_credentials",
        client_id: authAccount.godClientId,
        client_secret: authAccount.godClientSecret,
        audience
    }).then(({ data: { access_token: accessToken } }) => {
        instance.defaults.headers.common.Authorization = `Bearer ${accessToken}`;
        instance.defaults.baseURL = audience;
        return instance;
    }, err => {
        console.error(`Failed to retrieve oath token for domain: ${authAccount.domain} with clientId: ${authAccount.godClientId}.`, err);
        throw new Error("Failed retrieving management credentials.");
    });
});

// return /(dfio_user_login_delete_)[0-9]+(@test-netapp.com)/i.test(user.email);
// return /(dfio_)[0-9]+(@netapp-test.com)/i.test(user.email);
// return /(dfio_)[0-9]+(@netapp.com)/i.test(user.email);

const deleteDfioUsers = () => {
    return clients.auth0Client.getUsers('email,user_id')
        .then(users => {
            return users.filter(user => {
                return /(dfio_)[0-9]+(@netapp.com)/i.test(user.email);
            });
        })
        .then(dfioUsers => {
            logger.info(`${dfioUsers.length} dfio users to delete`);
            return Promise.map(dfioUsers, async user => {
                await deleteOccmClientsCreatedBy({userId: user.user_id});
                await deleteUser({userId: user.user_id, occms: null});
            }, {concurrency: 1});
        })
        .catch(console.log);
};

const deleteOccmClientsCreatedBy = ({ userId }) => {
    console.log(`Attempting to delete all occm clients created by ${userId}`);
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const userInfoParams = {
        TableName: process.env.userInfoTable,
        Key: { userId }
    };

    return dynamoDb.get(userInfoParams).promise().then(userInfoResponse => {
        const userInfo = userInfoResponse.Item || {};
        const occms = _.get(userInfo, "associatedOccms.values", []);

        if (!occms.length) {
            console.log('No associated occms to delete');
            return false;
        }

        return deleteOccms({ occms, userId }).then(() => {
            console.log("Removing occm clientId's from associated occms list");
            const dynamoDbOld = new AWS.DynamoDB();
            const params = {
                TableName: process.env.userInfoTable,
                ExpressionAttributeValues: {
                    ":c": { SS: occms }
                },
                Key: {
                    userId: {
                        S: userId
                    }
                },
                UpdateExpression: "DELETE associatedOccms :c"
            };

            return dynamoDbOld.updateItem(params).promise().then(() => true);
        });
    }).then(hadOccms => {
        if (hadOccms) { console.log("Occm clients deleted and associations removed"); }
        return null;
    })
        .catch(err => {
            console.log("Error deleting clients");
            throw err;
        });
};

const deleteOccms = ({ occms, userId }) => {
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    return authAxios().then(client => {
        const requestParams = {
            RequestItems: {
                [process.env.occmClientInfoTable]: {
                    Keys: _.take(occms, 100).map(clientId => // batchGet is limited to 100
                        ({ clientId }))
                }
            }
        };
        if (occms.length === 0) {
            return false;
        }
        return dynamoDb.batchGet(requestParams).promise().then(batchGetResponse => {
            const responseArr = _.get(batchGetResponse, `Responses.${process.env.occmClientInfoTable}`, []);
            return Promise.all(responseArr.map(resp => {
                if (resp.setupUserId === userId) { // check if authorized to delete client
                    const clientParams = {
                        TableName: process.env.occmClientInfoTable,
                        Key: { clientId: resp.clientId }
                    };
                    return dynamoDb.delete(clientParams).promise()// delete from dynamo
                        .then(() => client.delete(`/clients/${resp.clientId}`)) // delete from auth0
                        .catch(err => {
                            console.log(`Failed to delete client with Id: ${resp.clientId}`);
                            console.log(err);
                        });
                }

                console.log(`Not deleting client ${resp.clientId}. Setup user ${resp.setupUserId} does not match`);
                return null;
            })).then(() => true);
        });
    }).then(hadOccms => {
        if (hadOccms) { console.log("Deleted occm clients"); }
        return true;
    }).catch(err => {
        console.log("Failed to delete occm clients and dynamo data");
        console.log(err);
    });
};

const deleteUser = ({ userId, occms }) => {
    console.log(`Attempting to delete user with id: ${userId} and associated occms: ${occms}`);
    return deleteOccms({
        occms: occms || [],
        userId
    }).then(() => authAxios().then(client => client.delete(`users/${userId}`).then(({data}) => {
        console.log(data);
    })
        .then(() => deleteDynamoEntriesForUser({userId})))).then(() => {
        console.log(`User deletion successful`);
        return null;
    }, err => {
        console.error(`User deletion failed`, err);
        throw err;
    });
};

const deleteDynamoEntriesForUser = exports.deleteDynamoEntriesForUser = ({ userId }) => {
    // if (process.env.isTest) { return Promise.resolve(); }
    console.log(`Deleting dynamo entries for user ${userId}`);
    const dynamoDb = new AWS.DynamoDB.DocumentClient();
    const userInfoParams = {
        TableName: process.env.userInfoTable,
        Key: {
            userId: `${userId}`
        }
    };

    const deleteUserInfoP = dynamoDb.delete(userInfoParams).promise().catch(err => {
        console.log("Failed to delete user info entry");
        console.log(err);
    });

    const userPromoParams = {
        TableName: process.env.userPromoTable,
        KeyConditionExpression: 'userId = :userId',
        ExpressionAttributeValues: {
            ':userId': userId.toString()
        }
    };

    const userPromosP = dynamoDb.query(userPromoParams).promise().then(({ Items: userPromos }) => Promise.all(userPromos.map(userPromo => {
        const deleteParams = {
            TableName: process.env.userPromoTable,
            Key: {
                userId: userPromo.userId,
                promoId: userPromo.promoId
            }
        };

        return dynamoDb.delete(deleteParams).promise().catch(err => {
            console.log("Failed to delete userPromo entry");
            console.log(err);
        });
    }))).catch(err => {
        console.log("Failed to delete userPromos entries");
        console.log(err);
    });

    const userNotificationsParams = {
        TableName: process.env.notificationsTable,
        KeyConditionExpression: 'userId = :userId',
        ExpressionAttributeValues: {
            ':userId': userId.toString()
        }
    };

    const userNotificationsP = dynamoDb.query(userNotificationsParams).promise().then(({ Items: userNotifications }) => Promise.all(userNotifications.map(notification => {
        const deleteParams = {
            TableName: process.env.notificationsTable,
            Key: {
                userId: notification.userId,
                createDate: notification.createDate
            }
        };

        return dynamoDb.delete(deleteParams).promise().catch(err => {
            console.log("Failed to delete user notifications");
            console.log(err);
        });
    }))).catch(err => {
        console.log("Failed to delete user notifications");
        console.log(err);
    });

    return Promise.all([deleteUserInfoP, userPromosP, userNotificationsP]).catch(err => {
        console.log("Failed to clear dynamo entries for user");
        console.log(err);
    });
};

deleteDfioUsers().catch(console.log);
