const expect = require('chai').expect;
const Constants = require('../Constants');
const Auth0Client = require('./Auth0Client');
const TestLogger = require('../../logger/TestLogger');
const chai = require("chai");
const chaiAsPromised = require("chai-as-promised");
chai.use(chaiAsPromised);
const Joi = require('joi');

const logger = new TestLogger("Auth0Client-spec");
const auth0Client = new Auth0Client(logger);

describe('Auth0Client getCloudCentralUserToken', async function() {
    it('Should return only the access token string and not the whole data object when requesting for an auth0 token', async function() {
        const response = await auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        expect(typeof response).to.equal('string');
        expect(response.access_token).to.equal(undefined);
    });

    it('Should print an error message and return error object when credentials are not correct', async function() {
        const promise = auth0Client.getCloudCentralUserToken(Constants.defaultTestUserEmail, "wrongPassword");
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.error_description).to.equal('Wrong email or password.');
                expect(error.response.status).to.equal(403);
            })
    });
});

describe('Auth0Client getCloudSyncUserToken', async function() {
    it('Should return only the access token string and not the whole data object when requesting for an auth0 token', async function() {
        const response = await auth0Client.getCloudSyncUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        expect(typeof response).to.equal('string');
        expect(response.access_token).to.equal(undefined);
    });

    it('Should print an error message and return error object when credentials are not correct', async function() {
        const promise = auth0Client.getCloudSyncUserToken(Constants.defaultTestUserEmail, "wrongPassword");
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.error_description).to.equal('Wrong email or password.');
                expect(error.response.status).to.equal(403);
            })
    });
});

describe('Auth0Client getGovernanceUserToken', async function() {
    it('Should return only the access token string and not the whole data object when requesting for an auth0 token', async function() {
        const response = await auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, Constants.defaultTestUserPassword);
        expect(typeof response).to.equal('string');
        expect(response.access_token).to.equal(undefined);
    });

    it('Should print an error message and return error object when credentials are not correct', async function() {
        const promise = auth0Client.getGovernanceUserToken(Constants.defaultTestUserEmail, "wrongPassword");
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.error_description).to.equal('Wrong email or password.');
                expect(error.response.status).to.equal(403);
            })
    });
});

describe('Auth0Client getOccmRegistratorToken', async function() {
    it('Should return only the access token string and not the whole data object when requesting for an auth0 token', async function() {
        const response = await auth0Client.getOccmRegistratorToken();
        expect(typeof response).to.equal('string');
        expect(response.access_token).to.equal(undefined);
    });
});

describe('Auth0Client getOccmToken', async function() {
    it('Should print an error message and return error object when credentials are not correct', async function() {
        const promise = auth0Client.getOccmToken("blaI6Q4PvGAYAvKfhCpnqilqzPoQsLwD", "V3_blaiZXNVmrYQe6yqXMwW8OPEvYIJk3Y2RaMinF01LCrEkxboiJ0lfLH7B4J78");
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.error_description).to.equal('Unauthorized');
                expect(error.response.status).to.equal(401);
            })
    });
});

describe('Auth0Client getUsersByQuery', async function() {
    it('Should return a list of users details, according to optional filter and requested fields', async function() {
        const response = await auth0Client.getUsersByQuery(`email:${Constants.defaultTestUserEmail}`, 'email');
        expect(response).to.eql([{email: Constants.defaultTestUserEmail}]);
    });

    it('Should return an array of empty objects in case the requested field does not exist ', async function() {
        const response = await auth0Client.getUsersByQuery(`email:${Constants.defaultTestUserEmail}`, 'fake_field');
        expect(response).to.eql([{}]);
    });

    it('Should return an empty array in case the user that fit the filter was not found', async function() {
        const response = await auth0Client.getUsersByQuery(`email:fakeuser@fake.com`, 'email');
        expect(response).to.eql([]);
    });
});

describe('Auth0Client getUsers', async function() {
    it('Should return a list of clients details with requested fields', async function() {
        const schema = Joi.object().keys({
            name: Joi.string().required(),
            tenant:Joi.string().required()
        });
        const clients = await auth0Client.getClients('name');
        // noinspection BadExpressionStatementJS
        expect(clients[0]).to.not.be.empty;
        await Joi.validate(clients[0], schema);
    });

    it('Should return an error in case the requested field does not exist ', async function() {
        const promise = auth0Client.getClients('fake_field');
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.message.includes('Query validation error: \'String \'fake_field\' does not match pattern. Must be a comma separated list of the following values:')).to.equal(true);
                expect(error.response.status).to.equal(400);
            })
    });
});

describe('Auth0Client getClients', async function() {
    it('Should return a list of clients details with requested fields', async function() {
        const schema = Joi.object().keys({
            name: Joi.string().required(),
            tenant:Joi.string().required()
        });
        const clients = await auth0Client.getClients('name');
        // noinspection BadExpressionStatementJS
        expect(clients[0]).to.not.be.empty;
        await Joi.validate(clients[0], schema);
    });

    it('Should return an error in case the requested field does not exist ', async function() {
        const promise = auth0Client.getClients('fake_field');
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.message.includes('Query validation error: \'String \'fake_field\' does not match pattern. Must be a comma separated list of the following values:')).to.equal(true);
                expect(error.response.status).to.equal(400);
            })
    });
});

describe('Auth0Client getUserById', async function() {
    it('Should return user details for existing user', async function () {
        const response = await auth0Client.getUserById(Constants.defaultUserAuth0Sub);
        // noinspection BadExpressionStatementJS
        expect(response).to.not.be.empty;
        expect(response.email).to.equal(Constants.defaultTestUserEmail);
    });

    it('Should return 404 if user does not exist', async function () {
        const promise = auth0Client.getUserById('auth0|bla06c2876e57719a8248bla');
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.message).to.equal('The user does not exist.');
                expect(error.response.status).to.equal(404);
            })
    });

    it('Should return 400 if the user id is not well formed', async function () {
        const promise = auth0Client.getUserById('fakeUserId');
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.message).to.equal(`Object didn't pass validation for format user-id: fakeUserId`);
                expect(error.response.status).to.equal(400);
            })
    });
});

describe('Auth0Client updateUserDetails', async function() {
    it('Should return 400 if the body object is empty', async function () {
        const promise = auth0Client.updateUserDetails(Constants.defaultUserAuth0Sub, {});
        return expect(promise)
            .to.eventually.be.rejected
            .and.be.an.instanceOf(Error)
            .then(error => {
                expect(error.response.data.message.includes('Payload validation error')).to.equal(true);
                expect(error.response.status).to.equal(400);
            })
    });

    it('Should add/remove user property in app_metadata', async function () {
        const app_metadata = {
            "app_metadata" : {
                "cc:course-admin": true,
                "cc:clients": true,
                "cc:support": true
            }
        };
        let userDetails = await auth0Client.updateUserDetails(Constants.defaultUserAuth0Sub, app_metadata);
        expect(userDetails.app_metadata).to.eql(app_metadata.app_metadata);
        userDetails = await auth0Client.updateUserDetails(Constants.defaultUserAuth0Sub, {"app_metadata": {}});
        expect(userDetails.app_metadata).to.equal(undefined);
    });
});