const _ = require('lodash');
const {EC2} = require('aws-sdk');
const awaitUtil = require('../await-response-utils');
const processError = require('../axios-response-handler').processErrorObject;

/**
 * To get familiar with various optional request parameters:
 * https://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/EC2.html
 * **/

// TODO: add request objects
// TODO: error processing is lacking
class EC2Client {
    constructor(logger, awsOptions = {}) {
        this.ec2 = new EC2();
        this.logger = logger;
    }

    enhanceAwsErrorAndThrow(error, functionInput) {
        error.extraInfo = {invalidInput: functionInput};
        error.name = "AWS client exception: " + error.name;
        processError(error, this.logger);
        throw error;
    }

    describeInstances(params = {}) {
        return this.ec2.describeInstances(params).promise()
            .then(instanceResponse => {
                return instanceResponse.Reservations;
            })
            .catch(error => this.enhanceAwsErrorAndThrow(error, params));
    }

    deleteInstancesById(instancesIds) {
        return this.ec2.terminateInstances({InstanceIds: instancesIds}).promise();
    }

    describeVolumes(params = {}) {
        return this.ec2.describeVolumes(params).promise()
            .then(volumeResponse => {
                return volumeResponse.Volumes;
            })
            .catch(error => this.enhanceAwsErrorAndThrow(error, params));
    }

    deleteVolumeById(volumeId) {
        return this.ec2.deleteVolume({VolumeId: volumeId}).promise();
    };


    describeSnapshots(params = {}) {
        return this.ec2.describeSnapshots(params).promise()
            .then(snapshotResponse => {
                return snapshotResponse.Snapshots;
            })
            .catch(error => this.enhanceAwsErrorAndThrow(error, params));
    }

    deleteSnapshotById(snapshotId){
        return this.ec2.deleteSnapshot({SnapshotId: snapshotId}).promise();
    }

    getInstanceState(instanceId) {
        const params = {
            InstanceIds: [
                instanceId
            ]
        };
        return this.describeInstances(params)
            .then(instanceResponse => {
                return _.isEmpty(instanceResponse[0].Instances[0]) ? Promise.reject(new Error(`Could not find data for instanceId: ${instanceId}`)) : instanceResponse[0].Instances[0].State.Name;
            })
    }

    getSnapshotState(snapshotId) {
        const params = {
            SnapshotIds: [
                snapshotId
            ]
        };
        return this.describeSnapshots(params)
            .then(snapshotResponse => {
                return _.isEmpty(snapshotResponse[0]) ? Promise.reject(new Error(`Could not find data for snapshotId: ${snapshotId}`)) : snapshotResponse[0].State;
            })
    }

    getVolumeState(volumeId) {
        const params = {
            VolumeIds: [
                volumeId
            ]
        };
        return this.describeVolumes(params)
            .then(volumeResponse => {
                return _.isEmpty(volumeResponse[0]) ? Promise.reject(new Error(`Could not find data for snapshotId: ${volumeId}`)) : volumeResponse[0].State;
            })
    }

    createAvailableVolume(params) {
        return this.ec2.createVolume(params).promise()
            .then(volumeResponse => {
                const predicate = () => {
                    return this.getVolumeState(volumeResponse.VolumeId)
                        .then(state => {
                            if(_.isEqual(JSON.stringify(state), JSON.stringify('available'))) {
                                return volumeResponse;
                            } else {
                                return Promise.reject();
                            }
                        })
                };
                return awaitUtil.waitUntilComplete(predicate, 30000, 1000);
            })
    }

    createCompletedSnapshot(params) {
        return this.ec2.createSnapshot(params).promise()
            .then(snapshotResponse => {
                const predicate = () => {
                    return this.getSnapshotState(snapshotResponse.SnapshotId)
                        .then(state => {
                            if(_.isEqual(JSON.stringify(state), JSON.stringify('completed'))) {
                                return snapshotResponse;
                            } else {
                                return Promise.reject();
                            }
                        })
                };
                return awaitUtil.waitUntilComplete(predicate, 30000, 1000);
            })
    }

    createRunningInstance(params) {
        return this.ec2.runInstances(params).promise()
            .then(instanceResponse => {
                const predicate = () => {
                    return this.getInstanceState(instanceResponse.Instances[0].InstanceId)
                        .then(state => {
                            if (_.isEqual(JSON.stringify(state), JSON.stringify('running'))) {
                                return instanceResponse;
                            } else {
                                return Promise.reject();
                            }
                        })
                };
                return awaitUtil.waitUntilComplete(predicate, 60000, 1000);
            });
    }
}

module.exports = EC2Client;