const Constants = require('../Constants');
const jwtDecoder = require('../general-utils').decodeToken;
const governanceAxios = require('./cloudCentralAxios').governanceAxios();

class governanceClient {

    constructor(logger) {
        this.logger = logger;
    }

    deleteUser(userToken, occmsClientIds = []) {
        this.logger.info(`Deleting user ${jwtDecoder(userToken).sub} and optional associated occms ${occmsClientIds} from ${Constants.cloudCentralGovernanceApiBaseUrl}/delete-user`);
        return governanceAxios.post(`/delete-user`, {occms: occmsClientIds}, {
            headers: {
                Authorization: `Bearer ${userToken}`
            }
        })
    }

    deleteAssociatedOccms(govUserToken) {
        this.logger.info(`Deleting user ${jwtDecoder(govUserToken).sub} associated occm from ${Constants.cloudCentralGovernanceApiBaseUrl}/delete-associated-occms`);
        return governanceAxios.post(`/delete-associated-occms`, null, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        })
    }

    getClients(govUserToken) {
        this.logger.info(`Getting auth0 clients for user ${jwtDecoder(govUserToken).sub} from ${Constants.cloudCentralGovernanceApiBaseUrl}/developers/clients`);
        return governanceAxios.get(`/developers/clients`, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        })
    }

    getClient(govUserToken, clientId) {
        this.logger.info(`Getting auth0 client ${clientId} detail for user ${jwtDecoder(govUserToken).sub} from ${Constants.cloudCentralGovernanceApiBaseUrl}/developers/clients/${clientId}`);
        return governanceAxios.get(`/developers/clients/${clientId}`, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        })
    }

    updateClient(govUserToken, clientId, URIs) {
        this.logger.info(`Updating auth0 client ${clientId} detail for user ${jwtDecoder(govUserToken).sub} from ${Constants.cloudCentralGovernanceApiBaseUrl}/developers/clients/${clientId}`);
        return governanceAxios.patch(`/developers/clients/${clientId}`, null, {
            data: {
                allowedCallbackUrls: URIs.allowedCallbackUrls,
                allowedWebOrigins: URIs.allowedWebOrigins,
                allowedLogoutUrls: URIs.allowedLogoutUrls,
                allowedCors: URIs.allowedCors
            },
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        })
    }

    resetPassword(supervisorGovToken, subordinateEmail, subordinateNewPassword) {
        this.logger.info(`Updating ${subordinateEmail} password on behalf of user ${jwtDecoder(supervisorGovToken).sub} from ${Constants.cloudCentralGovernanceApiBaseUrl}/course/reset-password`);
        return governanceAxios.post(`/course/reset-password`, null, {
            data: {
                subordinateEmail: subordinateEmail,
                subordinateNewPassword: subordinateNewPassword
            },
            headers: {
                Authorization: `Bearer ${supervisorGovToken}`
            }
        })
    }

    getAllFederationConnections(govUserToken, fields = "") {
        this.logger.info(`Retrieving all federation connections for user ${jwtDecoder(govUserToken).sub}`);
        const fieldsQueryString = fields ? `?fields=${fields}` : "";
        return governanceAxios.get(`/connections${fieldsQueryString}`, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        });
    }

    enableFederationConnection(govUserToken, connectionName, companyName, emailDomains) {
        this.logger.info(`Enabling federation connection ${connectionName} by user ${jwtDecoder(govUserToken).sub}`);
        return governanceAxios.post(`/enable-connection`, {
            connectionName,
            companyName,
            emailDomains
        }, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        });
    }

    disableFederationConnection(govUserToken, connectionName) {
        this.logger.info(`Disabling federation connection ${connectionName} by user ${jwtDecoder(govUserToken).sub}`);
        return governanceAxios.post(`/disable-connection`, {connectionName}, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        });
    }

    getBlocks(govUserToken, userId) {
        this.logger.info(`Getting auth0 blocks client for user ${jwtDecoder(govUserToken).sub} from ${Constants.cloudCentralGovernanceApiBaseUrl}/support/users/${userId}/blocks`);
        return governanceAxios.get(`/support/users/${userId}/blocks`, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        });
    }

    deleteUserBlock(govUserToken, userId){
        this.logger.info(`Deleting auth0  blocks clients for user ${jwtDecoder(govUserToken).sub} from ${Constants.cloudCentralGovernanceApiBaseUrl}/support/users/${userId}/blocks`);
        return governanceAxios.delete(`/support/users/${userId}/blocks`, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        });
    }

    passwordChangeTicket(govUserToken, userId){
        this.logger.info(`User ${jwtDecoder(govUserToken).sub} is requesting reset password URL for user ${userId}`);
        return governanceAxios.post(`/support/users/${userId}/password-change-ticket`, null, {
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        });
    }

    getLogs(govUserToken, userId, page, perPage){
        this.logger.info(`User ${jwtDecoder(govUserToken).sub} is requesting Auth0 logs of user ${userId}`);
        return governanceAxios.get(`/support/users/${userId}/logs?page=${page}&perPage=${perPage}`,{
            headers: {
                Authorization: `Bearer ${govUserToken}`
            }
        });
    }
}

module.exports = governanceClient;
