import React from 'react';
import { connect } from 'react-redux';
import classNames from "classnames";
import { ReactComponent as CogIcon } from 'ui-components/src/images/monochrome-icons/ic-cog.svg';
import "./Accounts.scss"
import { AgGridReact } from 'ag-grid-react';
import 'ag-grid-community/dist/styles/ag-grid.css';
import 'ag-grid-community/dist/styles/ag-theme-balham.css';
import { ReactComponent as CloseXIcon } from 'ui-components/src/images/monochrome-icons/x-icon.svg';
import { ReactComponent as AccountAdminIcon } from "ui-components/src/images/monochrome-icons/ic-account-admin.svg";
import { ReactComponent as DotDotDotIcon } from "ui-components/src/images/monochrome-icons/ic-more.svg";
import { ReactComponent as CaretDownIcon } from 'ui-components/src/images/monochrome-icons/ic-caret-down.svg';
import Loader from "../../Loader";
import RemoveUserOverlay from './RemoveUserOverlay';
import ManageWorkspacesOverlay from './ManageWorkspacesOverlay';
import AddUserOverlay from './AddUserOverlay';
import ManageUserWorkspacesOverlay from './ManageUserWorkspacesOverlay';
import buttons from 'ui-components/src/button/button.module.scss';
import { history } from "../../store";
import querystring from 'query-string';
import accountsManagementMiddleware from "./../../middlewares/featureApis/accountsManagementMiddleware"
import { registerFeature } from "../../middlewares/apiMiddleware";

registerFeature("ACCOUNTS-MANAGEMENT", accountsManagementMiddleware);

const RoleCellRenderer = props => (
    <div className="role-cell">{props.data.userRole === 'Role-1' ?
        <span><AccountAdminIcon/>Account Admin</span> : 'Workspace Admin'}</div>
);
const generateWorkspacesDisplayString = (workspaces, role) => {
    if (!workspaces.length && role === 'Role-1') return (<span>All Workspaces</span>);
    if (!workspaces.length && role === 'Role-2') return (<span>None</span>);
    if (workspaces.length === 1) return (<span className="first-workspace-name">{workspaces[0].workspaceName}</span>);

    return (<span>
        <span className="first-workspace-name">{workspaces[0].workspaceName}</span>
        <span>&nbsp;{` + ${workspaces.length - 1}`}</span>
        <CaretDownIcon/>
    </span>);
};

const WorkspacesCellRenderer = props => (
    <div className="workspaces-cell"
         style={{cursor: props.data.workspaces.length > 1 ? 'pointer' : ''}}
         onClick={() => (props.data.workspaces.length > 1) && props.context.openAssociatedWorkspacesPopUp({
             user: props.data,
             rowIndex: props.rowIndex
         })}>
        {generateWorkspacesDisplayString(props.data.workspaces, props.data.userRole)}
    </div>
);

const MenuCellRenderer = props => (
    <div key={props.data.email} className="menu-cell">
        <button className="open-menu-button"
                onClick={() => props.context.openUserMenu({ user: props.data, rowIndex: props.rowIndex })}>
            <DotDotDotIcon/>
        </button>
    </div>
);

const getColumnDefs = () => {
    const fields = ["Name", "Email", "Role", "Workspaces"];
    const cellRenderersMap = {
        Role: "roleCellRenderer",
        Workspaces: "workspacesCellRenderer"
    };

    const columnDefs = fields.map(field => ({
        headerName: field,
        field: field.toLowerCase(),
        suppressMenu: true,
        unSortIcon: true,
        sortable: true,
        cellRenderer: cellRenderersMap[field]
    }));
    columnDefs.push({
        headerName: "",
        field: "Name",
        cellRenderer: "menuCellRenderer",
        colId: 'button'
    });

    return columnDefs;
};

const rowHeight = 55; // px

const UserPopUpMenu = ({user, rowIndex, isLastUserInAccount, closeUserMenu, removeUserPrompt, manageUserWorkspacesPrompt}) => (
    <div className="popup-overlay" onClick={() => closeUserMenu()}>
        <div className="popup user-popup-menu" style={{top: 235 + (rowHeight * rowIndex)}}>
            <div className="user-popup-menu-item">
                <button disabled={isLastUserInAccount} onClick={() => removeUserPrompt({ user })}>Remove User</button>
                {isLastUserInAccount &&
                <div className="user-popup-menu-item-popover">Last account admin cannot be deleted</div>}
            </div>
            {user.userRole === "Role-2" && <div className="user-popup-menu-item">
                <button onClick={() => manageUserWorkspacesPrompt({ user })}>Manage Workspaces</button>
            </div>}
        </div>
    </div>
);

const AssociatedWorkspacesPopUp = ({workspacesList, rowIndex, closeAssociatedWorkspacesPopUp}) => (
    <div className="popup-overlay" onClick={() => closeAssociatedWorkspacesPopUp()}>
        <div className="popup associated-workspaces-popup" style={{top: 235 + (rowHeight * rowIndex)}}>
            {workspacesList.map(workspace => (
                <div key={workspace.workspacePublicId}>{workspace.workspaceName}</div>))}
        </div>
    </div>);

class AccountsManagement extends React.Component {
    constructor(props) {
        super(props);

        this.columnDefs = getColumnDefs();
        const search = querystring.parse(window.location.search);

        if (search.accountId) {
            props.setAccountId({ accountId: search.accountId });
        }

        window.onmessage = function (e) {
            if (e.data.type === 'SET-ACCOUNT-ID') {
                history.replace(`/components/account?accountId=${e.data.accountId}`);
                props.setAccountId(e.data.payload);
            } else if (e.data.type === 'SET-TENANCY-BASE-URL') {
                props.setTenancyBaseUrl(e.data.payload);
            }
        };

        window.top.postMessage({ message: 'accountManagementReady' }, '*');
    }

    onFirstDataRendered(params) {
        this.api = params.api;
        params.api.sizeColumnsToFit();
    }

    onComponentStateChanged() {
        this.api.refreshCells({force: true});
    }

    closeButtonClicked() {
        if (this.props.currentUserMenu ||
            this.props.currentUserWorkspacesPopUp ||
            this.props.userForRemoval ||
            this.props.userForWorkspacesManagement ||
            this.props.manageWorkspaces ||
            this.props.addUserOverlayState
        ) this.props.closeAllOverlays();
        else this.props.closeAccountWidget()
    }

    render() {
        return <div className="accounts-management">
            <div className="accounts-header">
                <CogIcon/>
                <h3 className="main-title">Account Settings</h3>
                <div className="left-section">
                    <button className="manage-account-button"
                            onClick={this.props.openManageWorkspaces}
                            disabled={this.props.loadingData || this.props.accountManagementError}>
                        Manage Workspaces
                    </button>
                    <button className={classNames(buttons.primary, "associate-user-button")}
                            onClick={() => this.props.setAddUserOverlayState({ newState: 'email' })}
                            disabled={this.props.loadingData || this.props.accountManagementError}>+ Associate User
                    </button>
                    <button className="close-button" onClick={() => this.closeButtonClicked()}><CloseXIcon/></button>
                </div>
            </div>
            <div className="accounts-body">
                <h4>{this.props.account && `${this.props.account.accountName} users`}</h4>
                {this.props.loadingData && <Loader/>}
                {this.props.currentUserMenu &&
                    /*TODO: position with ref , not with rowheight*/
                <UserPopUpMenu user={this.props.currentUserMenu.user}
                               rowIndex={this.props.currentUserMenu.rowIndex}
                               isLastUserInAccount={this.props.accountUsers.length === 1}
                               closeUserMenu={this.props.closeUserMenu}
                               removeUserPrompt={this.props.removeUserPrompt}
                               manageUserWorkspacesPrompt={this.props.manageUserWorkspacesPrompt}/>
                }
                {this.props.currentUserWorkspacesPopUp && <AssociatedWorkspacesPopUp
                    workspacesList={this.props.currentUserWorkspacesPopUp.user.workspaces}
                    rowIndex={this.props.currentUserWorkspacesPopUp.rowIndex}
                    closeAssociatedWorkspacesPopUp={this.props.closeAssociatedWorkspacesPopUp}/>}
                {this.props.accountManagementError &&
                <div className="main-view-error-message">{this.props.accountManagementError}</div>}
                <div className="ag-theme-balham portal-ag-grid">
                    {this.props.accountUsers && <AgGridReact
                        enableColResize={false}
                        columnDefs={this.columnDefs}
                        onFirstDataRendered={this.onFirstDataRendered.bind(this)}
                        onComponentStateChanged={this.onComponentStateChanged.bind(this)}
                        rowData={this.props.accountUsers}
                        rowHeight={rowHeight}
                        headerHeight={rowHeight}
                        suppressColumnVirtualisation="true"
                        frameworkComponents={{
                            menuCellRenderer: MenuCellRenderer,
                            roleCellRenderer: RoleCellRenderer,
                            workspacesCellRenderer: WorkspacesCellRenderer
                        }}
                        context={{
                            openUserMenu: this.props.openUserMenu,
                            openAssociatedWorkspacesPopUp: this.props.openAssociatedWorkspacesPopUp,
                            currentUserMenu: this.props.currentUserMenu
                        }}
                    >
                    </AgGridReact>}
                </div>
                {this.props.userForRemoval && <RemoveUserOverlay/>}
                {this.props.userForWorkspacesManagement && <ManageUserWorkspacesOverlay/>}
                {this.props.manageWorkspaces && <ManageWorkspacesOverlay/>}
                {this.props.addUserOverlayState && <AddUserOverlay/>}
            </div>
        </div>;
    }
}

const mapStateToProps = state => {
    return {
        loadingData: state.accountsManagement.loadingData,
        account: state.accountsManagement.account,
        accountUsers: state.accountsManagement.accountUsers,
        manageWorkspaces: state.accountsManagement.manageWorkspaces,
        accountManagementError: state.accountsManagement.accountManagementError,
        currentUserMenu: state.accountsManagement.currentUserMenu,
        currentUserWorkspacesPopUp: state.accountsManagement.currentUserWorkspacesPopUp,
        userForRemoval: state.accountsManagement.userForRemoval,
        userForWorkspacesManagement: state.accountsManagement.userForWorkspacesManagement,
        addUserOverlayState: state.accountsManagement.addUserOverlayState
    };
};

const mapDispatchToProps = dispatch => {
    return {
        setTenancyBaseUrl: ({ tenancyBaseUrl }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:SET-TENANCY-BASE-URL",
                payload: { tenancyBaseUrl }
            });
        },
        setAccountId: ({ accountId }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:SET-ACCOUNT-ID",
                payload: { accountId }
            });
        },
        openManageWorkspaces: () => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:OPEN-MANAGE-WORKSPACES",
            });
        },
        openUserMenu: ({ user, rowIndex }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:OPEN-USER-MENU",
                payload: { user, rowIndex }
            })
        },
        closeUserMenu: () => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:CLOSE-USER-MENU"
            })
        },
        openAssociatedWorkspacesPopUp: ({ user, rowIndex }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:OPEN-ASSOCIATED-WORKSPACES-POP-UP",
                payload: { user, rowIndex }
            })
        },
        closeAssociatedWorkspacesPopUp: () => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:CLOSE-ASSOCIATED-WORKSPACES-POP-UP"
            })
        },
        removeUserPrompt: ({ user }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:REMOVE-USER-PROMPT",
                payload: user
            })
        },
        manageUserWorkspacesPrompt: ({ user }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-PROMPT",
                payload: user
            })
        },
        setAddUserOverlayState: ({ newState }) => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:SET-ADD-USER-OVERLAY-STATE",
                payload: newState
            })
        },
        closeAllOverlays: () => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:CLOSE-ALL-OVERLAYS"
            })
        },
        closeAccountWidget: () => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:CLOSE-ACCOUNT-WIDGET"
            })
        }
    }
};

export default connect(mapStateToProps, mapDispatchToProps)(AccountsManagement)
