import React from 'react';
import styles from "./Dashboard.module.scss"
import classNames from "classnames";
import _ from "lodash";
import connect from "react-redux/es/connect/connect";
import Chart from 'react-apexcharts'
import { ReactComponent as PieChartIcon } from 'ui-components/src/images/monochrome-icons/ic-piechart.svg';
import { ReactComponent as FilesIcon } from 'ui-components/src/images/monochrome-icons/ic-files.svg';
import { ReactComponent as CvoLogo } from 'ui-components/src/images/netapp-services/ic-cvo.svg';
import { ReactComponent as CvsLogo } from 'ui-components/src/images/netapp-services/ic-cvs.svg';
import { ReactComponent as AzureBlobIcon } from 'ui-components/src/images/protocol-endpoints-icons/azure-blob.svg';
import { ReactComponent as CifsIcon } from 'ui-components/src/images/protocol-endpoints-icons/cifs.svg';
import { ReactComponent as EfsIcon } from 'ui-components/src/images/protocol-endpoints-icons/efs.svg';
import { ReactComponent as GcpIcon } from 'ui-components/src/images/protocol-endpoints-icons/gcp.svg';
import { ReactComponent as IbmStorageIcon } from 'ui-components/src/images/protocol-endpoints-icons/ibm-storage.svg';
import { ReactComponent as NfsIcon } from 'ui-components/src/images/protocol-endpoints-icons/nfs.svg';
import { ReactComponent as S3Icon } from 'ui-components/src/images/protocol-endpoints-icons/s3.svg';
import { ReactComponent as StorageGridIcon } from 'ui-components/src/images/protocol-endpoints-icons/storage-grid.svg';

const Datum = ({ Icon, figure, unit }) => <div className={styles.datum}>
    {Icon && <Icon/>}
    <span className={classNames(styles.figure, (figure === undefined) && styles.loading)}>{figure || ' '}</span>
    <span className={classNames(styles.unit, (figure === undefined) && styles.loading)}>{unit || ' '}</span>
</div>;

const CloudSyncProtocolIcon = ({ Icon, protocolName, count }) =>
    <div className={classNames(styles.protocol, !count && styles.disabled)}>
        <Icon/>
        {(count || '') && <div className={styles['count-badge']}>{count}</div>}
        <div className={styles['protocol-name']}>{protocolName}</div>
    </div>;

const protocols = [
    { protocol: 'nfs', protocolName: 'NFS Server', Icon: NfsIcon },
    { protocol: 'cifs', protocolName: 'CIFS Server', Icon: CifsIcon },
    { protocol: 'efs', protocolName: 'AWS EFS', Icon: EfsIcon },
    { protocol: 's3', protocolName: 'AWS S3', Icon: S3Icon },
    { protocol: 'azure', protocolName: 'Azure Blob', Icon: AzureBlobIcon },
    { protocol: 'sgws', protocolName: 'Storage Grid', Icon: StorageGridIcon },
    { protocol: 'ibm', protocolName: 'IBM Object Storage', Icon: IbmStorageIcon },
    { protocol: 'gcp', protocolName: 'Google Cloud', Icon: GcpIcon },
];

class CloudSyncActivatedDashboard extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            options: {
                chart: {
                    fontFamily: "'Open Sans', sans-serif",
                    foreColor: '#7b7b7b',
                    toolbar: {
                        show: false
                    }
                },
                xaxis: {
                    categories: this.props.cloudSyncData.dataTransferredByDay.map(d => d.date),
                    labels: {
                        formatter: x => x.slice(-2), // Takes 2 last chars of date, this is the day section. '2019-03-22' => '22'
                    },
                    title: {
                        text: "Data transferred in the last 30 days",
                        style: {
                            cssClass: 'cloudsync-chart-xaxis-title',
                        },
                    },
                },
                yaxis: {
                    labels: {
                        formatter: y => `${y} ${this.props.cloudSyncData.highestNormalizedCapacityUnitInLast30days}`, // For example '3 GiB'
                    }
                },
                tooltip: {
                    x: { formatter: x => x }
                },
                dataLabels: {
                    enabled: false
                },
            },
            series: [{
                name: 'Transferred successfully',
                data: []
            }]
        };

        this.props.fetchData();
    }

    componentDidUpdate(prevProps, prevState, snapshot) {
        if ((!prevProps.cloudSyncData && this.props.cloudSyncData) ||
            (prevProps.cloudSyncData && this.props.cloudSyncData && !_.isEqual(prevProps.cloudSyncData.dataTransferredByDay, this.props.cloudSyncData.dataTransferredByDay))) {
            this.setState({
                options: {
                    ...this.state.options,
                    xaxis: {
                        ...this.state.options.xaxis,
                        categories: this.props.cloudSyncData.dataTransferredByDay.map(d => d.date),
                    },
                    yaxis: {
                        labels: {
                            formatter: y => `${y} ${this.props.cloudSyncData.highestNormalizedCapacityUnitInLast30days}`, // For example '3 GiB'
                        }
                    }
                },
                series: [{
                    name: 'Transferred successfully',
                    data: this.props.cloudSyncData.dataTransferredByDay.map(d => d.capacityCopied)
                }]
            });
        }
    };


    render() {
        const {
            totalBytes,
            totalFiles,
            cvoRelationships,
            cvsRelationships,
            protocolsCount
        } = this.props.cloudSyncData || {};

        return <div>
            {this.props.cloudSyncError &&
            <div className={styles['error-message']}>Could not retrieve Cloud Sync usage
                data: {this.props.cloudSyncError.toString()}</div>}
            {this.props.cloudSyncData && <article className={styles['cloud-sync-dashboard']}>
                <div className={styles['data-card']}>
                    <div className={styles["card-header"]}>Total Data Transferred</div>
                    <div className={styles["card-body"]}>
                        <div className={styles["datum-title"]}>Total Amount Ever Transferred Successfully Over All
                            Relationships
                        </div>
                        <Datum Icon={PieChartIcon} figure={totalBytes.capacity} unit={totalBytes.unit}/>
                        <Datum Icon={FilesIcon} figure={totalFiles} unit={"Files"}/>
                    </div>
                </div>
                <div className={styles['data-card']}>
                    <div className={styles["card-header"]}>Service Integrations</div>
                    <div className={classNames(styles["card-body"], styles['service-integrations'])}>
                        <div>
                            <div className={styles["datum-title"]}>Cloud Volumes Ontap</div>
                            <Datum Icon={CvoLogo} figure={cvoRelationships} unit={"Relationships"}/>
                        </div>
                        <div>
                            <div className={styles["datum-title"]}>Cloud Volumes Service</div>
                            <Datum Icon={CvsLogo} figure={cvsRelationships} unit={"Relationships"}/>
                        </div>
                    </div>
                </div>
                <div className={classNames(styles['data-card'], styles['full-row-card'])}>
                    <div className={styles["card-header"]}>Data Transferred Successfully</div>
                    <div className={styles["card-body"]}>
                        <div className={styles["datum-title"]}>Total Amount Transferred Successfully Per Day</div>
                        <Chart className={styles.chart} options={this.state.options} series={this.state.series}
                               type="bar" width={1055} height={175}/>
                    </div>
                </div>
                <div className={classNames(styles['data-card'], styles['full-row-card'])}>
                    <div className={styles["card-header"]}>Protocols & Object Storage Providers</div>
                    <div className={styles["card-body"]}>
                        <div className={styles["datum-title"]}>Total Amount Transferred Successfully Per Day</div>
                        <div className={styles.protocols}>
                            {protocols.map(protocol =>
                                <CloudSyncProtocolIcon key={protocol.protocol}
                                                       protocol={protocol.protocol}
                                                       protocolName={protocol.protocolName}
                                                       Icon={protocol.Icon}
                                                       count={protocolsCount[protocol.protocol]}/>
                            )}
                        </div>
                    </div>
                </div>
            </article>}
        </div>
    }
}

const mapStateToProps = state => {
    return {
        cloudSyncData: state.dashboards.cloudSyncData,
        cloudSyncDataLoading: state.dashboards.cloudSyncDataLoading,
        cloudSyncError: state.dashboards.cloudSyncError
    };
};

const mapDispatchToProps = dispatch => {
    return {
        fetchData: () => {
            dispatch({
                type: "DASHBOARDS:FETCH-CLOUD-SYNC-DATA"
            });
        }
    }
};

export default connect(mapStateToProps, mapDispatchToProps)(CloudSyncActivatedDashboard);
