import React from 'react'
import { connect } from 'react-redux'
import _ from 'lodash'
import "./FabricView.scss"
import {registerFeature} from './../../middlewares/apiMiddleware';
import fabricViewMiddleware from "./../../middlewares/featureApis/fabricViewMiddleware";
import classNames from 'classnames';
import styles from "./FabricView.module.scss";
import * as actions from "../../actions";

import { ReactComponent as AwsLogo } from 'ui-components/src/images/third-party-services/ic-aws.svg';
import { ReactComponent as GcpLogo } from 'ui-components/src/images/third-party-services/ic-gcp.svg';
import { ReactComponent as OnPremLogo } from 'ui-components/src/images/third-party-services/ic-on-prem.svg';
import { ReactComponent as AzureLogo } from 'ui-components/src/images/third-party-services/ic-microsoft.svg';

import Transition from 'react-transition-group/Transition';

import ExternalLink from "../widgets/ExternalLink";
import CloudCentralSecondaryHeader from "../headers/CloudCentralSecondaryHeader";
import CloudVolumesDialogHandler from "./../launch-pad/CloudVolumesDialogHandler";
import CloudVolumesOntapHandler from "./../launch-pad/CloudVolumesOntapHandler";

import { ReactComponent as OutlineIcon } from 'ui-components/src/images/netapp-services/ic-outline.svg';

registerFeature("FABRIC-VIEW", fabricViewMiddleware);

const FabricViewHeader = () => <CloudCentralSecondaryHeader title="Fabric View"/>;

const FabricTableHeader = () =>
    <div className="table-header-container nav-adjusted-fixed-pane">
        <div className="table-header">
            <div className="first-column"/>
            <div className="provider-title">
                <AzureLogo className="azure"/>
                Microsoft Azure
            </div>
            <div className="provider-title">
                <AwsLogo className="aws"/>
                Amazon Web Services</div>
            <div className="provider-title">
                <GcpLogo className="gcp"/>
                Google Cloud Platform</div>
            <div className="provider-title">
                <OnPremLogo className="onprem"/>On-Premises</div>
        </div>
    </div>;

const DataCell = ({state, value, unit, text}) => {
    if (state === "loading") {
        return <div className="data-cell">
            &nbsp;
            <div className="capacity-mask"/>
            <div className="capacity-mask-cover"/>
            <div className="capacity-text-mask"/>
        </div>
    } else {
        if (value) {
            return <div className="data-cell">&nbsp;
                <span className="amount">
                    <span className="value">{value}</span>
                    <span className="unit">{unit}</span>
                </span>
                <span className="text">{text}</span>
            </div>
        } else {
            return <div className="no-data-cell">&nbsp;</div>;
        }
    }
};

const ServiceRow = ({fadeStatus, launchPadService, service: {key, title, data, description, Logo, className, noCta}}) => {
    const {status, text, serviceKey} = launchPadService;

    return<div className={classNames("service-row", styles[fadeStatus], styles['service-row'])}>
     <div className="title-column">
        &nbsp;
        <Logo/>
        <span className={classNames("title", {noLink: noCta})}>{title}</span>
        {!noCta &&
            status === actions.STATUS_PENDING ?
                <div className={styles['cta-mask']}/> :
                text ? <ExternalLink linkKey={`launchPad.services.${serviceKey}`} trackCategory="fabricView" className="link-group link">{text}</ExternalLink> : ''
        }
        {key === "cloudVolumesOntap" && <CloudVolumesOntapHandler trackCategory="fabricView" view={launchPadService.view} data={launchPadService.data}/>}
        {key === "cloudVolumesService" && <CloudVolumesDialogHandler origin="fabricView" className="link-group"/>}
    </div>
    {data && <div className="data-row">
        <DataCell {...data.azure}/>
        <DataCell {...data.aws}/>
        <DataCell {...data.gcp}/>
        <DataCell {...data.onprem}/>
    </div>}
    {!data && <div className="no-data-row">&nbsp;<span>{description}</span></div>}
</div>};

const EmptyServiceRow = ({index}) => <div className="service-row">
    <div className="title-column">
        &nbsp;
        <OutlineIcon className={styles['outline-shimmer']}/>
        <div className={styles['title-mask']}/>
        <div className={styles['cta-mask']}/>
    </div>
    <div className="no-data-row">&nbsp;
        <div className={`${styles['description-mask']} index-${index % 5}`}/>
    </div>
</div>;

const FabricTable = ({servicesShowCount, servicesLoaded, fabricViewServices, launchPadServices, servicesCount}) => {
    return <div className="fabric-table">
        {servicesLoaded && fabricViewServices.map((service, index) => {
            return <Transition in={servicesShowCount > index} timeout={1} key={service.key}>
                {(fadeStatus) => {
                    if(fadeStatus === 'exited') return <EmptyServiceRow index={index} key={index}/>;
                    return <ServiceRow fadeStatus={fadeStatus} launchPadService={launchPadServices[service.key] || {}} service={service}/>
                }}
            </Transition>
        })}
        {!servicesLoaded && servicesCount && _.range(servicesCount).map((index) =>
            <EmptyServiceRow index={index} key={index}/>
        )}
    </div>
};

const FabricView = ({fabricViewServices, initialize, servicesLoaded, launchPadServices, servicesCount, servicesShowCount}) => {
    initialize();
    return <div className="fabric-view">
        <div>
            <FabricViewHeader/>
            <FabricTableHeader/>
            <FabricTable servicesShowCount={servicesShowCount} servicesLoaded={servicesLoaded} servicesCount={servicesCount} launchPadServices={launchPadServices} fabricViewServices={fabricViewServices}/>
        </div>
  </div>;
};

const initOnce = _.once(f => f());
const mapDispatchToProps = dispatch => {
    return {
        initialize: () => {
            initOnce(() => {
                setTimeout(() => dispatch({
                    type: "FABRIC-VIEW:INITIALIZE"
                }));
            });
        }
    }
};

const mapStateToProps = state => {
    return {
        launchPadServices: state.services,
        fabricViewServices: state.fabricView.services,
        servicesLoaded: state.fabricView.servicesLoaded,
        servicesCount: state.fabricView.servicesCount,
        servicesShowCount: state.fabricView.servicesShowCount
    }
};

export default connect(mapStateToProps, mapDispatchToProps)(FabricView)