import React, {Component} from 'react';
import styles from "./SpiffAccounts.module.scss"
import {AgGridReact} from "ag-grid-react";
import _ from "lodash";
import SearchWidget from "./../../widgets/SearchWidget.js"
import RowCounter from "./../../widgets/RowCounter.js"
import TableWidgetRow from "./../../widgets/TableWidgetRow.js"
import connect from "react-redux/es/connect/connect";
import Loader from "../../../Loader";
import DownloadWidget from "./../../../components/widgets/DownloadWidget"
import { ReactComponent as InfoIcon } from "ui-components/src/images/monochrome-icons/ic-info-i.svg";
import {openDialog} from './../../dialog/Dialog';
import { ReactComponent as CloseXIcon } from "ui-components/src/images/monochrome-icons/x-icon.svg";
import { ReactComponent as BulletChevron } from 'ui-components/src/images/monochrome-icons/bullet-chevron-icon.svg';

const getColumnDefs = () => {
    const fields = ["Account Name", "Sales Rep Name", "DM/Sales Rep Manager", "Solutions Engineer", "Segment", "Geo", "Wave"];
    const initialColumnsWidth = {
        'Account Name': 340,
        'Sales Rep Name': 180,
        'DM/Sales Rep Manager': 180,
        "Segment": 90,
        "Geo": 90,
        "Wave": 80
    };

    return _.map(fields, (field => ({
        headerName: field,
        field: field,
        width: initialColumnsWidth[field],
        sortable: true,
        suppressMenu: true,
        unSortIcon: true,
        autoHeight: true
    })));
};

const LearnMoreDialog = ({closeDialog}) => <div>
    <header>
        Accounts
        <button onClick={closeDialog}><CloseXIcon/></button>
    </header>
    <div className={styles["dialog-content"]}>
        <p>The Active IQ program team identified the accounts listed here as users of one or more of the following technologies via the collection of ASUP data that NetApp receives. Because the accounts listed have met this criteria, these accounts are good prospects for CVO for DR.</p>
        <ul>
            <li><BulletChevron/>AltaVault/SnapVault</li>
            <li><BulletChevron/>SnapMirror</li>
            <li><BulletChevron/>Has 1 or more NetApp storage devices installed (FAS, AFF, Eseries etc.)</li>
            <li><BulletChevron/>Uses PowerShell and other automation tools.</li>
        </ul>
    </div>
</div>;

class DumbAccountsTable extends Component {
    constructor(props) {
        super(props);

        this.state = {quickFilter: ""};
        this.columnDefs = getColumnDefs();

        this.quickFilterChanged = this.quickFilterChanged.bind(this);
    }

    onFirstDataRendered(params) {
        this.api = params.api;
        params.api.sizeColumnsToFit();
        params.api.resetRowHeights();
        this.setState({show: true})
    }

    onComponentStateChanged() {
        this.api.refreshCells({force: true});
    }

    onFilterChanged() {
        this.setState({filteredCount: this.api.getModel().rootNode.childrenAfterFilter.length});
    }

    quickFilterChanged(value) {
        this.setState({quickFilter: value});
        this.api.setQuickFilter(value);
    }

    render(){
        const { status, accounts, errorMessage } = this.props;
        return <div className={styles.container}>
            <div>
                <button className={styles['learn-more-button']} onClick={el => openDialog(<LearnMoreDialog className={styles['learn-more-dialog']}/>, el.target)}>
                    Learn more about these accounts <InfoIcon/>
                </button>
            </div>
            {(status === "PENDING" || !this.state.show) && <Loader/>}
            {status === "ERROR" && <div>
                {errorMessage}
            </div>}
            {status === "SUCCESS" && <React.Fragment>
                <TableWidgetRow>
                    <RowCounter filteredCount={this.state.filteredCount} totalCount={accounts.length} singularTitle="Account" pluralTitle="Accounts"/>
                    <SearchWidget setFilter={this.quickFilterChanged.bind(this)}/>
                    <DownloadWidget onClick={this.props.downloadAccounts}/>
                </TableWidgetRow>
                <div className={`ag-theme-balham ${styles.table} ${!this.state.show ? 'transparent-hide' : ''}`}>
                    <AgGridReact
                        enableColResize={false}
                        columnDefs={this.columnDefs}
                        onFirstDataRendered={this.onFirstDataRendered.bind(this)}
                        onComponentStateChanged={this.onComponentStateChanged.bind(this)}
                        onFilterChanged={this.onFilterChanged.bind(this)}
                        rowData={accounts}
                        headerHeight="55"
                        floatingFiltersHeight="55"
                        suppressColumnVirtualisation="true">
                    </AgGridReact>
                </div>
            </React.Fragment>}
        </div>
    }
}

const mapStateToProps = state => {
    return {
        status: state.internal.americasGlobalSpiffStatus,
        accounts: state.internal.americasGlobalSpiffData.accounts,
        errorMessage: state.internal.americasGlobalSpiffError
    };
};

const mapDispatchToProps = dispatch => {
    return {
        downloadAccounts: () => {
            dispatch({
                type: "INTERNAL:DOWNLOAD-AMERICAS-GLOBAL-ACCOUNTS"
            })
        }
    }
};

const AccountsTable = connect(mapStateToProps, mapDispatchToProps)(DumbAccountsTable);

export default () => <div className={styles.base}>
    <AccountsTable/>
</div>;