import React from 'react'
import {connect} from 'react-redux'
import "./CustomerSuccess.scss"
import _ from "lodash"
import {AgGridReact} from 'ag-grid-react';
import 'ag-grid-community/dist/styles/ag-grid.css';
import 'ag-grid-community/dist/styles/ag-theme-balham.css';
import {Dialog} from "../../dialog/Dialog";
import { CloseIcon } from 'components/svg-icons/Icons';
import EnumFloatingFilter from "./FloatingFilter";
import ExternalLink from "../../widgets/ExternalLink";
import { ReactComponent as NEArrowIcon } from 'ui-components/src/images/monochrome-icons/ne-arrow-icon.svg';
import classNames from "classnames";
import Loader from "../../../Loader";
import { ReactComponent as DocumentIcon } from 'ui-components/src/images/2-colors-icons/ic-document.svg';
import { ReactComponent as SalesBellIcon } from "ui-components/src/images/2-colors-icons/ic-sales-bell.svg";
import { ReactComponent as CaseStudyIcon } from "ui-components/src/images/2-colors-icons/ic-case-study.svg";
import { ReactComponent as SalesEnablementIcon } from "ui-components/src/images/2-colors-icons/ic-sales-enablement.svg";
import { ReactComponent as VideoIcon } from "ui-components/src/images/2-colors-icons/ic-video.svg";
import { ReactComponent as WebinarIcon } from "ui-components/src/images/2-colors-icons/ic-webinar.svg";
import { ReactComponent as PressReleaseIcon } from 'ui-components/src/images/2-colors-icons/ic-press-release.svg';

const UploadMessageDumb = ({uploadMessage, uploadMessageType}) => {
    if(uploadMessage) {
        return <div className={classNames("upload-message", uploadMessageType)}>{uploadMessage}</div>
    } else {
        return null;
    }
};

const mapStateToPropsUploadMessage = state => {
    return {
        uploadMessage: state.customerSuccess.uploadMessage,
        uploadMessageType: state.customerSuccess.uploadMessageType
    };
};

const UploadMessage = connect(mapStateToPropsUploadMessage)(UploadMessageDumb);

const linkLabelToIcon = (label) => {
    switch (label) {
        case "Sales Bell Slide":
            return <SalesBellIcon/>;
        case "Case Study":
            return <CaseStudyIcon/>;
        case "Sales Enablement":
            return <SalesEnablementIcon/>;
        case "Video":
            return <VideoIcon/>;
        case "Webinar":
            return <WebinarIcon/>;
        case "Press Release":
            return <PressReleaseIcon/>;
        default:
            return <DocumentIcon/>
    }
};

const DisplayLinksDialog = ({closeDialog, data}) => <div>
    <header>
        <button className="close-button" onClick={closeDialog}>
            <CloseIcon/>
        </button>
        <h3>{data.Company}</h3>
    </header>
    <div className='dialog-content'>
        {_.map(data, (value, key) => (value && _.startsWith(key, 'LinkName')) ?
            <ExternalLink key={key} title={value} href={data[key.replace('LinkName', 'LinkUrl')]}
                          linkKey={`customerSuccessStories.${data.Company}-${value}`} newTab>
                {linkLabelToIcon(value)}
                <div>{value}</div>
            </ExternalLink> : null)}
    </div>
</div>;

class ExternalLinkRenderer extends React.Component {
    constructor(props) {
        super(props);

        this.state = { open: false }
    }

    refresh(params) {
        this.setState({
            open: params.context.linksDialogOpen === this.props.data.Company
        });

        return true;
    }

    numOfExternalLinks(data) {
        return _.filter(data, (value, key) => (value && _.startsWith(key, 'LinkUrl'))).length;
    }

    numOfButtonLines(data) {
        return _.ceil(this.numOfExternalLinks(data) / 3);
    }

    render() {
        return <div key={this.props.data.Company}>
            <button className="open-links-dialog-button" ref={el => this.button = el}
                    onClick={() => this.props.context.openDialog(this.props.data.Company)}>
                <NEArrowIcon/>
            </button>
            <Dialog target={this.button} open={this.state.open}
                    closeTrigger={this.props.context.closeTrigger}>
                <DisplayLinksDialog data={this.props.data}
                                    className={`display-links-dialog dialog-${this.numOfButtonLines(this.props.data)}-lines`}/>
            </Dialog>
        </div>;
    }
}

const getColumnDefs = () => {
    const fields = ["Company", "Vertical", "Region", "Country", "Use Case", "Product", "Cloud Provider", "Public Reference"];
    const initialColumnsWidth = {
        'Country': 106,
        'Vertical': 147,
        'Region': 108,
        'Use Case': 195,
        'Product': 195,
        'Cloud Provider': 105,
        'Public Reference': 144
    };
    const columnDefs = _.map(fields, (field => ({
        headerName: field,
        field: field,
        width: initialColumnsWidth[field],
        filter: "agTextColumnFilter",
        floatingFilterComponent: field === "Company" ? "" : "enumFloatingFilter",
        floatingFilterComponentParams: {
            suppressFilterButton: true
        },
        suppressMenu: true,
        sortable: true,
        unSortIcon: true
    })));
    columnDefs.push({
        headerName: "Display Links",
        field: "Company",
        width: 105,
        cellRenderer: "externalLinkRenderer",
        colId: 'button'
    });

    return columnDefs;
};

class CustomerSuccess extends React.Component {
    constructor(props) {
        super(props);
        props.fetchData();

        this.columnDefs = getColumnDefs();
    }

    uploadFile(event) {
        const file = event.target.files[0];
        this.props.uploadFile(file);
    }

    onFirstDataRendered(params) {
        this.api = params.api;
        params.api.sizeColumnsToFit();
    }

    onComponentStateChanged() {
        this.api.refreshCells({force: true});
    }

    onFilterChanged() {
        this.props.filterChanged(this.api.getModel().rootNode.childrenAfterFilter.length);
    }

    render() {
        return <div className="customer-success">
            <h4>Customer Success</h4>

            <UploadMessage/>

            {(this.props.uploading || !this.props.customerSuccessData) && <Loader/>}

            <div className="table-top">
                {typeof this.props.numberOfFilteredRows === 'number' && this.props.customerSuccessData && this.props.customerSuccessData.asJson && this.props.numberOfFilteredRows !== this.props.customerSuccessData.asJson.length && `${(this.props.numberOfFilteredRows || 0)} of `}
                {this.props.customerSuccessData && this.props.customerSuccessData.asJson && this.props.customerSuccessData.asJson.length} Stories

                {this.props.userHasCustomerStoriesScope && <div className="upload-button-wrapper">
                    <button className="upload-button">Upload a file</button>
                    <input type="file" name="myfile" onChange={el => this.uploadFile(el)}/>
                </div>}
                <button onClick={() => this.props.downloadAsCsv({data: this.props.customerSuccessData.asCsv})}>
                    Download as CSV
                </button>
            </div>
            {this.props.customerSuccessData && <div
                className="ag-theme-balham portal-ag-grid"
                style={{
                    height: '500px',
                    width: '100%'
                }}>
                <AgGridReact
                    enableColResize={false}
                    columnDefs={this.columnDefs}
                    floatingFilter={true}
                    onFirstDataRendered={this.onFirstDataRendered.bind(this)}
                    onComponentStateChanged={this.onComponentStateChanged.bind(this)}
                    onFilterChanged={this.onFilterChanged.bind(this)}
                    rowData={this.props.customerSuccessData.asJson}
                    rowHeight="55"
                    headerHeight="55"
                    floatingFiltersHeight="55"
                    suppressColumnVirtualisation="true"
                    frameworkComponents={{
                        enumFloatingFilter: EnumFloatingFilter,
                        externalLinkRenderer: ExternalLinkRenderer
                    }}
                    context={{
                        openDialog: this.props.openLinksDialog,
                        closeTrigger: this.props.closeLinksDialog,
                        linksDialogOpen: this.props.linksDialogOpen,
                    }}>
                </AgGridReact>
            </div>}
        </div>;
    }
}

const mapStateToProps = state => {
    return {
        customerSuccessData: state.customerSuccess.customerSuccessData,
        linksDialogOpen: state.customerSuccess.linksDialogOpen,
        numberOfFilteredRows: state.customerSuccess.numberOfFilteredRows,
        uploading: state.customerSuccess.uploading,
        userHasCustomerStoriesScope: state.auth.scopes["cc:customer-stories"],
    };
};

const mapDispatchToProps = dispatch => {
    return {
        fetchData: () => {
            dispatch({
                type: "CUSTOMER-SUCCESS:FETCH-STORIES",
            })
        },
        downloadAsCsv: (args) => {
            dispatch({
                type: "CUSTOMER-SUCCESS:DOWNLOAD-AS-CSV",
                payload: args
            })
        },
        uploadFile: (file) => {
            dispatch({
                type: "CUSTOMER-SUCCESS:UPLOAD-FILE",
                payload: file
            })
        },
        openLinksDialog: (companyName) => {
            dispatch({
                type: "CUSTOMER-SUCCESS:OPEN-LINKS-DIALOG",
                payload: companyName
            })
        },
        closeLinksDialog: () => {
            dispatch({
                type: "CUSTOMER-SUCCESS:CLOSE-LINKS-DIALOG"
            })
        },
        filterChanged: (numberOfFilteredRows) => {
            dispatch({
                type: "CUSTOMER-SUCCESS:FILTER-CHANGED",
                payload: numberOfFilteredRows
            })
        }

    }
};

export default connect(mapStateToProps, mapDispatchToProps)(CustomerSuccess)
