import axios from 'axios'
import _ from 'lodash'
import links from "../../consts/links";

let tenancyAxios;

const getTenancyAxios = () => {
    if (tenancyAxios) return tenancyAxios;

    tenancyAxios = axios.create({
        baseURL: links.launchPad.services.tenancyBackend,
        headers: {
            'X-User-Token': axios.defaults.headers.common['Authorization']
        }
    });

    return tenancyAxios;
};

const getErrorMessage = (error, action) => `Could not ${action}: ${(error.response && error.response.data) || error.toString()}`;
const closeWidget = () => {
    window.top.postMessage({ message: 'closeAccountManagementWidget' }, '*');
};

export default ({ dispatch, next, action, getState }) => {
    const { type, payload } = action;
    const accountId = _.get(getState(), 'accountsManagement.accountId');

    if (type === "ACCOUNTS-MANAGEMENT:SET-TENANCY-BASE-URL") {
        getTenancyAxios().defaults.baseURL = payload.tenancyBaseUrl;
    } else if (type === "ACCOUNTS-MANAGEMENT:SET-ACCOUNT-ID") {
        //Using timeout to make sure the accountId was updated in state before dispatching fetch-data:
        setTimeout(() => {
            dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-DATA" });
        })
    } else if (type === "ACCOUNTS-MANAGEMENT:FETCH-DATA") {
        Promise.all([
            getTenancyAxios().get(`/account/${accountId}/users`),
            getTenancyAxios().get(`/account/${accountId}/workspace`),
            getTenancyAxios().get(`/account`)
        ])
            .then(([{ data: users }, { data: workspaces }, { data: accounts }]) => {
                const account = _.find(accounts, { accountPublicId: accountId });
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-DATA-SUCCESS",
                    payload: { users, workspaces, account }
                });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-DATA-ERROR",
                    payload: { errorMessage: getErrorMessage(error, 'retrieve account data') }
                });
            });
        return next({ type: "ACCOUNTS-MANAGEMENT:FETCH-DATA-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:REMOVE-USER") {
        const { associatedUserPublicId } = payload;

        getTenancyAxios().post(`/account/${accountId}/disassociate`, { associatedUserPublicId })
            .then(() => {
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:REMOVE-USER-SUCCESS",
                        payload: { associatedUserPublicId }
                    });
                },
                error => {
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:REMOVE-USER-ERROR",
                        payload: { errorMessage: getErrorMessage(error, 'remove user') }
                    });
                });

        return next({ type: "ACCOUNTS-MANAGEMENT:REMOVE-USER-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES") {
        const { user, workspaceIdsToRemove, workspaceIdsToAdd } = payload;
        const addWorkspacesPromises = workspaceIdsToAdd.map(workspacePublicId => getTenancyAxios().post(`/account/${accountId}/workspace/${workspacePublicId}/add-user/${user.userPublicId}`));
        const removeWorkspacesPromises = workspaceIdsToRemove.map(workspacePublicId => getTenancyAxios().post(`/account/${accountId}/workspace/${workspacePublicId}/remove-user/${user.userPublicId}`));

        Promise.all(addWorkspacesPromises.concat(removeWorkspacesPromises)).then(() => {
            dispatch({ type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-SUCCESS" });
            dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-DATA" });
        }, error => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-ERROR",
                payload: { errorMessage: getErrorMessage(error, 'manage user workspaces') }
            })
        });
        return next({ type: "ACCOUNTS-MANAGEMENT:MANAGE-USER-WORKSPACES-PENDING" });
    } else if (type === "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME") {
        const { workspace, newName } = payload;

        getTenancyAxios().patch(`/account/${accountId}/workspace/${workspace.workspacePublicId}`, { newName })
            .then(() => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-SUCCESS"
                });
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:FETCH-DATA"
                });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-ERROR",
                    payload: { errorMessage: getErrorMessage(error, `rename workspace ${workspace.workspaceName}`) }
                });
            });

        return next({
            type: "ACCOUNTS-MANAGEMENT:EDIT-WORKSPACE-NAME-PENDING",
            payload: payload.workspace.workspacePublicId
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE") {
        const { workspacePublicId, workspaceName } = payload;

        getTenancyAxios().delete(`/account/${accountId}/workspace/${workspacePublicId}`)
            .then(() => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-SUCCESS",
                    payload
                });
                dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-DATA" });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-ERROR",
                    payload: { errorMessage: getErrorMessage(error, `delete workspace ${workspaceName}`) }
                });
            });

        return next({
            type: "ACCOUNTS-MANAGEMENT:DELETE-WORKSPACE-PENDING",
            payload: workspacePublicId
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE") {
        const { newName } = payload;

        getTenancyAxios().post(`/account/${accountId}/workspace/${newName}`)
            .then(({ data: newWorkspace }) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-SUCCESS",
                    payload: newWorkspace
                });
            }, error => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-ERROR",
                    payload: { errorMessage: getErrorMessage(error, `add new workspace ${newName}`) }
                });
            });

        return next({
            type: "ACCOUNTS-MANAGEMENT:ADD-NEW-WORKSPACE-PENDING"
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL") {
        axios.get("/portal/does-user-exist", { params: { email: payload.userEmail } })
            .then(({ data }) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-SUCCESS",
                    payload: { email: payload.userEmail, exists: data.doesExist }
                });
                if (data.doesExist) {
                    dispatch({
                        type: "ACCOUNTS-MANAGEMENT:SET-ADD-USER-OVERLAY-STATE",
                        payload: "role"
                    });
                }
            }, ({ response }) => {
                dispatch({
                    type: "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-ERROR",
                    payload: { errorMessage: `Could not check if user with email ${payload.userEmail} exists in Cloud Central: ${response.data.message}` }
                });
            });

        return next({
            type: "ACCOUNTS-MANAGEMENT:CHECK-IF-ASSOCIATED-EMAIL-EXISTS-IN-CLOUD-CENTRAL-PENDING"
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER") {
        const { associatedEmail, rolePublicId, workspaces } = payload;

        const errorCallback = error => {
            dispatch({
                type: "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-ERROR",
                payload: { errorMessage: getErrorMessage(error, `associate user`) }
            })
        };

        getTenancyAxios().post(`/account/${accountId}/associate`, { associatedEmail, rolePublicId })
            .then(({ data: user }) => {
                const associateUserToWorkspacesPromise = rolePublicId === "Role-2" ?
                    Promise.all(workspaces.map(workspace => getTenancyAxios()
                        .post(`/account/${accountId}/workspace/${workspace}/add-user/${user.userPublicId}`))) :
                    Promise.resolve();

                associateUserToWorkspacesPromise
                    .then(() => {
                        dispatch({
                            type: "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-SUCCESS",
                            payload: { user }
                        });
                        dispatch({ type: "ACCOUNTS-MANAGEMENT:FETCH-DATA" });

                    }, errorCallback)
            }, errorCallback);

        return next({
            type: "ACCOUNTS-MANAGEMENT:ASSOCIATE-USER-PENDING"
        });
    } else if (type === "ACCOUNTS-MANAGEMENT:CLOSE-ACCOUNT-WIDGET") {
        dispatch({ type: "ACCOUNTS-MANAGEMENT:CLOSE-ALL-OVERLAYS" });
        closeWidget();
    }
    return next(action);
};
